#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_wanopt_webcache
short_description: Configure global Web cache settings in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS (FOS) device by allowing the
      user to set and modify wanopt feature and webcache category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.5
version_added: "2.9"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
        description:
            - FortiOS or FortiGate IP address.
        type: str
        required: false
    username:
        description:
            - FortiOS or FortiGate username.
        type: str
        required: false
    password:
        description:
            - FortiOS or FortiGate password.
        type: str
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        type: str
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS protocol.
        type: bool
        default: true
    ssl_verify:
        description:
            - Ensures FortiGate certificate must be verified by a proper CA.
        type: bool
        default: true
    wanopt_webcache:
        description:
            - Configure global Web cache settings.
        default: null
        type: dict
        suboptions:
            always_revalidate:
                description:
                    - Enable/disable revalidation of requested cached objects, which have content on the server, before serving it to the client.
                type: str
                choices:
                    - enable
                    - disable
            cache_by_default:
                description:
                    - Enable/disable caching content that lacks explicit caching policies from the server.
                type: str
                choices:
                    - enable
                    - disable
            cache_cookie:
                description:
                    - Enable/disable caching cookies. Since cookies contain information for or about individual users, they not usually cached.
                type: str
                choices:
                    - enable
                    - disable
            cache_expired:
                description:
                    - Enable/disable caching type-1 objects that are already expired on arrival.
                type: str
                choices:
                    - enable
                    - disable
            default_ttl:
                description:
                    - Default object expiry time . This only applies to those objects that do not have an expiry time set by the web server.
                type: int
            external:
                description:
                    - Enable/disable external Web caching.
                type: str
                choices:
                    - enable
                    - disable
            fresh_factor:
                description:
                    - Frequency that the server is checked to see if any objects have expired (1 - 100). The higher the fresh factor, the less often the
                       checks occur.
                type: int
            host_validate:
                description:
                    - "Enable/disable validating Host: with original server IP."
                type: str
                choices:
                    - enable
                    - disable
            ignore_conditional:
                description:
                    - Enable/disable controlling the behavior of cache-control HTTP 1.1 header values.
                type: str
                choices:
                    - enable
                    - disable
            ignore_ie_reload:
                description:
                    - "Enable/disable ignoring the PNC-interpretation of Internet Explorer's Accept: / header."
                type: str
                choices:
                    - enable
                    - disable
            ignore_ims:
                description:
                    - Enable/disable ignoring the if-modified-since (IMS) header.
                type: str
                choices:
                    - enable
                    - disable
            ignore_pnc:
                description:
                    - Enable/disable ignoring the pragma no-cache (PNC) header.
                type: str
                choices:
                    - enable
                    - disable
            max_object_size:
                description:
                    - Maximum cacheable object size in kB (1 - 2147483 kb (2GB). All objects that exceed this are delivered to the client but not stored in
                       the web cache.
                type: int
            max_ttl:
                description:
                    - Maximum time an object can stay in the web cache without checking to see if it has expired on the server .
                type: int
            min_ttl:
                description:
                    - Minimum time an object can stay in the web cache without checking to see if it has expired on the server .
                type: int
            neg_resp_time:
                description:
                    - Time in minutes to cache negative responses or errors (0 - 4294967295).
                type: int
            reval_pnc:
                description:
                    - Enable/disable revalidation of pragma-no-cache (PNC) to address bandwidth concerns.
                type: str
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
   ssl_verify: "False"
  tasks:
  - name: Configure global Web cache settings.
    fortios_wanopt_webcache:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      wanopt_webcache:
        always_revalidate: "enable"
        cache_by_default: "enable"
        cache_cookie: "enable"
        cache_expired: "enable"
        default_ttl: "7"
        external: "enable"
        fresh_factor: "9"
        host_validate: "enable"
        ignore_conditional: "enable"
        ignore_ie_reload: "enable"
        ignore_ims: "enable"
        ignore_pnc: "enable"
        max_object_size: "15"
        max_ttl: "16"
        min_ttl: "17"
        neg_resp_time: "18"
        reval_pnc: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible.module_utils.network.fortios.fortios import FortiOSHandler
from ansible.module_utils.network.fortimanager.common import FAIL_SOCKET_MSG


def login(data, fos):
    host = data['host']
    username = data['username']
    password = data['password']
    ssl_verify = data['ssl_verify']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password, verify=ssl_verify)


def filter_wanopt_webcache_data(json):
    option_list = ['always_revalidate', 'cache_by_default', 'cache_cookie',
                   'cache_expired', 'default_ttl', 'external',
                   'fresh_factor', 'host_validate', 'ignore_conditional',
                   'ignore_ie_reload', 'ignore_ims', 'ignore_pnc',
                   'max_object_size', 'max_ttl', 'min_ttl',
                   'neg_resp_time', 'reval_pnc']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def underscore_to_hyphen(data):
    if isinstance(data, list):
        for elem in data:
            elem = underscore_to_hyphen(elem)
    elif isinstance(data, dict):
        new_data = {}
        for k, v in data.items():
            new_data[k.replace('_', '-')] = underscore_to_hyphen(v)
        data = new_data

    return data


def wanopt_webcache(data, fos):
    vdom = data['vdom']
    wanopt_webcache_data = data['wanopt_webcache']
    filtered_data = underscore_to_hyphen(filter_wanopt_webcache_data(wanopt_webcache_data))

    return fos.set('wanopt',
                   'webcache',
                   data=filtered_data,
                   vdom=vdom)


def is_successful_status(status):
    return status['status'] == "success" or \
        status['http_method'] == "DELETE" and status['http_status'] == 404


def fortios_wanopt(data, fos):

    if data['wanopt_webcache']:
        resp = wanopt_webcache(data, fos)

    return not is_successful_status(resp), \
        resp['status'] == "success", \
        resp


def main():
    fields = {
        "host": {"required": False, "type": "str"},
        "username": {"required": False, "type": "str"},
        "password": {"required": False, "type": "str", "default": "", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "ssl_verify": {"required": False, "type": "bool", "default": True},
        "wanopt_webcache": {
            "required": False, "type": "dict", "default": None,
            "options": {
                "always_revalidate": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "cache_by_default": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "cache_cookie": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "cache_expired": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "default_ttl": {"required": False, "type": "int"},
                "external": {"required": False, "type": "str",
                             "choices": ["enable", "disable"]},
                "fresh_factor": {"required": False, "type": "int"},
                "host_validate": {"required": False, "type": "str",
                                  "choices": ["enable", "disable"]},
                "ignore_conditional": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "ignore_ie_reload": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "ignore_ims": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "ignore_pnc": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "max_object_size": {"required": False, "type": "int"},
                "max_ttl": {"required": False, "type": "int"},
                "min_ttl": {"required": False, "type": "int"},
                "neg_resp_time": {"required": False, "type": "int"},
                "reval_pnc": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)

    # legacy_mode refers to using fortiosapi instead of HTTPAPI
    legacy_mode = 'host' in module.params and module.params['host'] is not None and \
                  'username' in module.params and module.params['username'] is not None and \
                  'password' in module.params and module.params['password'] is not None

    if not legacy_mode:
        if module._socket_path:
            connection = Connection(module._socket_path)
            fos = FortiOSHandler(connection)

            is_error, has_changed, result = fortios_wanopt(module.params, fos)
        else:
            module.fail_json(**FAIL_SOCKET_MSG)
    else:
        try:
            from fortiosapi import FortiOSAPI
        except ImportError:
            module.fail_json(msg="fortiosapi module is required")

        fos = FortiOSAPI()

        login(module.params, fos)
        is_error, has_changed, result = fortios_wanopt(module.params, fos)
        fos.logout()

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2012, Michael DeHaan <michael.dehaan@gmail.com>
# (c) 2016, Toshio Kuratomi <tkuratomi@ansible.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['stableinterface'],
                    'supported_by': 'core'}


DOCUMENTATION = '''
---
module: ping
version_added: historical
short_description: Try to connect to host, verify a usable python and return C(pong) on success.
description:
   - A trivial test module, this module always returns C(pong) on successful
     contact. It does not make sense in playbooks, but it is useful from
     C(/usr/bin/ansible) to verify the ability to login and that a usable python is configured.
   - This is NOT ICMP ping, this is just a trivial test module.
options: {}
author:
    - "Ansible Core Team"
    - "Michael DeHaan"
'''

EXAMPLES = '''
# Test we can logon to 'webservers' and execute python with json lib.
ansible webservers -m ping
'''

from ansible.module_utils.basic import AnsibleModule


def main():
    module = AnsibleModule(
        argument_spec=dict(
            data=dict(required=False, default=None),
        ),
        supports_check_mode=True
    )
    result = dict(ping='pong')
    if module.params['data']:
        if module.params['data'] == 'crash':
            raise Exception("boom")
        result['ping'] = module.params['data']
    result['location'] = 'library'
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Google
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file at
#     https://www.github.com/GoogleCloudPlatform/magic-modules
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function

__metaclass__ = type

################################################################################
# Documentation
################################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ["preview"], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: gcp_compute_health_check
description:
- Health Checks determine whether instances are responsive and able to do work.
- They are an important part of a comprehensive load balancing configuration, as they
  enable monitoring instances behind load balancers.
- Health Checks poll instances at a specified interval. Instances that do not respond
  successfully to some number of probes in a row are marked as unhealthy. No new connections
  are sent to unhealthy instances, though existing connections will continue. The
  health check will continue to poll unhealthy instances. If an instance later responds
  successfully to some number of consecutive probes, it is marked healthy again and
  can receive new connections.
short_description: Creates a GCP HealthCheck
version_added: 2.6
author: Google Inc. (@googlecloudplatform)
requirements:
- python >= 2.6
- requests >= 2.18.4
- google-auth >= 1.3.0
options:
  state:
    description:
    - Whether the given object should exist in GCP
    choices:
    - present
    - absent
    default: present
    type: str
  check_interval_sec:
    description:
    - How often (in seconds) to send a health check. The default value is 5 seconds.
    required: false
    default: '5'
    type: int
  description:
    description:
    - An optional description of this resource. Provide this property when you create
      the resource.
    required: false
    type: str
  healthy_threshold:
    description:
    - A so-far unhealthy instance will be marked healthy after this many consecutive
      successes. The default value is 2.
    required: false
    default: '2'
    type: int
  name:
    description:
    - Name of the resource. Provided by the client when the resource is created. The
      name must be 1-63 characters long, and comply with RFC1035. Specifically, the
      name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`
      which means the first character must be a lowercase letter, and all following
      characters must be a dash, lowercase letter, or digit, except the last character,
      which cannot be a dash.
    required: true
    type: str
  timeout_sec:
    description:
    - How long (in seconds) to wait before claiming failure.
    - The default value is 5 seconds. It is invalid for timeoutSec to have greater
      value than checkIntervalSec.
    required: false
    default: '5'
    type: int
    aliases:
    - timeout_seconds
  unhealthy_threshold:
    description:
    - A so-far healthy instance will be marked unhealthy after this many consecutive
      failures. The default value is 2.
    required: false
    default: '2'
    type: int
  type:
    description:
    - Specifies the type of the healthCheck, either TCP, SSL, HTTP or HTTPS. If not
      specified, the default is TCP. Exactly one of the protocol-specific health check
      field must be specified, which must match type field.
    - 'Some valid choices include: "TCP", "SSL", "HTTP", "HTTPS"'
    required: false
    type: str
  http_health_check:
    description:
    - A nested object resource.
    required: false
    type: dict
    suboptions:
      host:
        description:
        - The value of the host header in the HTTP health check request.
        - If left empty (default value), the public IP on behalf of which this health
          check is performed will be used.
        required: false
        type: str
      request_path:
        description:
        - The request path of the HTTP health check request.
        - The default value is /.
        required: false
        default: "/"
        type: str
      response:
        description:
        - The bytes to match against the beginning of the response data. If left empty
          (the default value), any response will indicate health. The response data
          can only be ASCII.
        required: false
        type: str
      port:
        description:
        - The TCP port number for the HTTP health check request.
        - The default value is 80.
        required: false
        type: int
      port_name:
        description:
        - Port name as defined in InstanceGroup#NamedPort#name. If both port and port_name
          are defined, port takes precedence.
        required: false
        type: str
      proxy_header:
        description:
        - Specifies the type of proxy header to append before sending data to the
          backend, either NONE or PROXY_V1. The default is NONE.
        - 'Some valid choices include: "NONE", "PROXY_V1"'
        required: false
        default: NONE
        type: str
      port_specification:
        description:
        - 'Specifies how port is selected for health checking, can be one of the following
          values: * `USE_FIXED_PORT`: The port number in `port` is used for health
          checking.'
        - "* `USE_NAMED_PORT`: The `portName` is used for health checking."
        - "* `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for
          each network endpoint is used for health checking. For other backends, the
          port or named port specified in the Backend Service is used for health checking."
        - If not specified, HTTP health check follows behavior specified in `port`
          and `portName` fields.
        - 'Some valid choices include: "USE_FIXED_PORT", "USE_NAMED_PORT", "USE_SERVING_PORT"'
        required: false
        type: str
        version_added: 2.9
  https_health_check:
    description:
    - A nested object resource.
    required: false
    type: dict
    suboptions:
      host:
        description:
        - The value of the host header in the HTTPS health check request.
        - If left empty (default value), the public IP on behalf of which this health
          check is performed will be used.
        required: false
        type: str
      request_path:
        description:
        - The request path of the HTTPS health check request.
        - The default value is /.
        required: false
        default: "/"
        type: str
      response:
        description:
        - The bytes to match against the beginning of the response data. If left empty
          (the default value), any response will indicate health. The response data
          can only be ASCII.
        required: false
        type: str
      port:
        description:
        - The TCP port number for the HTTPS health check request.
        - The default value is 443.
        required: false
        type: int
      port_name:
        description:
        - Port name as defined in InstanceGroup#NamedPort#name. If both port and port_name
          are defined, port takes precedence.
        required: false
        type: str
      proxy_header:
        description:
        - Specifies the type of proxy header to append before sending data to the
          backend, either NONE or PROXY_V1. The default is NONE.
        - 'Some valid choices include: "NONE", "PROXY_V1"'
        required: false
        default: NONE
        type: str
      port_specification:
        description:
        - 'Specifies how port is selected for health checking, can be one of the following
          values: * `USE_FIXED_PORT`: The port number in `port` is used for health
          checking.'
        - "* `USE_NAMED_PORT`: The `portName` is used for health checking."
        - "* `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for
          each network endpoint is used for health checking. For other backends, the
          port or named port specified in the Backend Service is used for health checking."
        - If not specified, HTTPS health check follows behavior specified in `port`
          and `portName` fields.
        - 'Some valid choices include: "USE_FIXED_PORT", "USE_NAMED_PORT", "USE_SERVING_PORT"'
        required: false
        type: str
        version_added: 2.9
  tcp_health_check:
    description:
    - A nested object resource.
    required: false
    type: dict
    suboptions:
      request:
        description:
        - The application data to send once the TCP connection has been established
          (default value is empty). If both request and response are empty, the connection
          establishment alone will indicate health. The request data can only be ASCII.
        required: false
        type: str
      response:
        description:
        - The bytes to match against the beginning of the response data. If left empty
          (the default value), any response will indicate health. The response data
          can only be ASCII.
        required: false
        type: str
      port:
        description:
        - The TCP port number for the TCP health check request.
        - The default value is 443.
        required: false
        type: int
      port_name:
        description:
        - Port name as defined in InstanceGroup#NamedPort#name. If both port and port_name
          are defined, port takes precedence.
        required: false
        type: str
      proxy_header:
        description:
        - Specifies the type of proxy header to append before sending data to the
          backend, either NONE or PROXY_V1. The default is NONE.
        - 'Some valid choices include: "NONE", "PROXY_V1"'
        required: false
        default: NONE
        type: str
      port_specification:
        description:
        - 'Specifies how port is selected for health checking, can be one of the following
          values: * `USE_FIXED_PORT`: The port number in `port` is used for health
          checking.'
        - "* `USE_NAMED_PORT`: The `portName` is used for health checking."
        - "* `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for
          each network endpoint is used for health checking. For other backends, the
          port or named port specified in the Backend Service is used for health checking."
        - If not specified, TCP health check follows behavior specified in `port`
          and `portName` fields.
        - 'Some valid choices include: "USE_FIXED_PORT", "USE_NAMED_PORT", "USE_SERVING_PORT"'
        required: false
        type: str
        version_added: 2.9
  ssl_health_check:
    description:
    - A nested object resource.
    required: false
    type: dict
    suboptions:
      request:
        description:
        - The application data to send once the SSL connection has been established
          (default value is empty). If both request and response are empty, the connection
          establishment alone will indicate health. The request data can only be ASCII.
        required: false
        type: str
      response:
        description:
        - The bytes to match against the beginning of the response data. If left empty
          (the default value), any response will indicate health. The response data
          can only be ASCII.
        required: false
        type: str
      port:
        description:
        - The TCP port number for the SSL health check request.
        - The default value is 443.
        required: false
        type: int
      port_name:
        description:
        - Port name as defined in InstanceGroup#NamedPort#name. If both port and port_name
          are defined, port takes precedence.
        required: false
        type: str
      proxy_header:
        description:
        - Specifies the type of proxy header to append before sending data to the
          backend, either NONE or PROXY_V1. The default is NONE.
        - 'Some valid choices include: "NONE", "PROXY_V1"'
        required: false
        default: NONE
        type: str
      port_specification:
        description:
        - 'Specifies how port is selected for health checking, can be one of the following
          values: * `USE_FIXED_PORT`: The port number in `port` is used for health
          checking.'
        - "* `USE_NAMED_PORT`: The `portName` is used for health checking."
        - "* `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for
          each network endpoint is used for health checking. For other backends, the
          port or named port specified in the Backend Service is used for health checking."
        - If not specified, SSL health check follows behavior specified in `port`
          and `portName` fields.
        - 'Some valid choices include: "USE_FIXED_PORT", "USE_NAMED_PORT", "USE_SERVING_PORT"'
        required: false
        type: str
        version_added: 2.9
extends_documentation_fragment: gcp
notes:
- 'API Reference: U(https://cloud.google.com/compute/docs/reference/rest/v1/healthChecks)'
- 'Official Documentation: U(https://cloud.google.com/load-balancing/docs/health-checks)'
'''

EXAMPLES = '''
- name: create a health check
  gcp_compute_health_check:
    name: test_object
    type: TCP
    tcp_health_check:
      port_name: service-health
      request: ping
      response: pong
    healthy_threshold: 10
    timeout_sec: 2
    unhealthy_threshold: 5
    project: test_project
    auth_kind: serviceaccount
    service_account_file: "/tmp/auth.pem"
    state: present
'''

RETURN = '''
checkIntervalSec:
  description:
  - How often (in seconds) to send a health check. The default value is 5 seconds.
  returned: success
  type: int
creationTimestamp:
  description:
  - Creation timestamp in RFC3339 text format.
  returned: success
  type: str
description:
  description:
  - An optional description of this resource. Provide this property when you create
    the resource.
  returned: success
  type: str
healthyThreshold:
  description:
  - A so-far unhealthy instance will be marked healthy after this many consecutive
    successes. The default value is 2.
  returned: success
  type: int
id:
  description:
  - The unique identifier for the resource. This identifier is defined by the server.
  returned: success
  type: int
name:
  description:
  - Name of the resource. Provided by the client when the resource is created. The
    name must be 1-63 characters long, and comply with RFC1035. Specifically, the
    name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`
    which means the first character must be a lowercase letter, and all following
    characters must be a dash, lowercase letter, or digit, except the last character,
    which cannot be a dash.
  returned: success
  type: str
timeoutSec:
  description:
  - How long (in seconds) to wait before claiming failure.
  - The default value is 5 seconds. It is invalid for timeoutSec to have greater value
    than checkIntervalSec.
  returned: success
  type: int
unhealthyThreshold:
  description:
  - A so-far healthy instance will be marked unhealthy after this many consecutive
    failures. The default value is 2.
  returned: success
  type: int
type:
  description:
  - Specifies the type of the healthCheck, either TCP, SSL, HTTP or HTTPS. If not
    specified, the default is TCP. Exactly one of the protocol-specific health check
    field must be specified, which must match type field.
  returned: success
  type: str
httpHealthCheck:
  description:
  - A nested object resource.
  returned: success
  type: complex
  contains:
    host:
      description:
      - The value of the host header in the HTTP health check request.
      - If left empty (default value), the public IP on behalf of which this health
        check is performed will be used.
      returned: success
      type: str
    requestPath:
      description:
      - The request path of the HTTP health check request.
      - The default value is /.
      returned: success
      type: str
    response:
      description:
      - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
      returned: success
      type: str
    port:
      description:
      - The TCP port number for the HTTP health check request.
      - The default value is 80.
      returned: success
      type: int
    portName:
      description:
      - Port name as defined in InstanceGroup#NamedPort#name. If both port and port_name
        are defined, port takes precedence.
      returned: success
      type: str
    proxyHeader:
      description:
      - Specifies the type of proxy header to append before sending data to the backend,
        either NONE or PROXY_V1. The default is NONE.
      returned: success
      type: str
    portSpecification:
      description:
      - 'Specifies how port is selected for health checking, can be one of the following
        values: * `USE_FIXED_PORT`: The port number in `port` is used for health checking.'
      - "* `USE_NAMED_PORT`: The `portName` is used for health checking."
      - "* `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the port
        or named port specified in the Backend Service is used for health checking."
      - If not specified, HTTP health check follows behavior specified in `port` and
        `portName` fields.
      returned: success
      type: str
httpsHealthCheck:
  description:
  - A nested object resource.
  returned: success
  type: complex
  contains:
    host:
      description:
      - The value of the host header in the HTTPS health check request.
      - If left empty (default value), the public IP on behalf of which this health
        check is performed will be used.
      returned: success
      type: str
    requestPath:
      description:
      - The request path of the HTTPS health check request.
      - The default value is /.
      returned: success
      type: str
    response:
      description:
      - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
      returned: success
      type: str
    port:
      description:
      - The TCP port number for the HTTPS health check request.
      - The default value is 443.
      returned: success
      type: int
    portName:
      description:
      - Port name as defined in InstanceGroup#NamedPort#name. If both port and port_name
        are defined, port takes precedence.
      returned: success
      type: str
    proxyHeader:
      description:
      - Specifies the type of proxy header to append before sending data to the backend,
        either NONE or PROXY_V1. The default is NONE.
      returned: success
      type: str
    portSpecification:
      description:
      - 'Specifies how port is selected for health checking, can be one of the following
        values: * `USE_FIXED_PORT`: The port number in `port` is used for health checking.'
      - "* `USE_NAMED_PORT`: The `portName` is used for health checking."
      - "* `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the port
        or named port specified in the Backend Service is used for health checking."
      - If not specified, HTTPS health check follows behavior specified in `port`
        and `portName` fields.
      returned: success
      type: str
tcpHealthCheck:
  description:
  - A nested object resource.
  returned: success
  type: complex
  contains:
    request:
      description:
      - The application data to send once the TCP connection has been established
        (default value is empty). If both request and response are empty, the connection
        establishment alone will indicate health. The request data can only be ASCII.
      returned: success
      type: str
    response:
      description:
      - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
      returned: success
      type: str
    port:
      description:
      - The TCP port number for the TCP health check request.
      - The default value is 443.
      returned: success
      type: int
    portName:
      description:
      - Port name as defined in InstanceGroup#NamedPort#name. If both port and port_name
        are defined, port takes precedence.
      returned: success
      type: str
    proxyHeader:
      description:
      - Specifies the type of proxy header to append before sending data to the backend,
        either NONE or PROXY_V1. The default is NONE.
      returned: success
      type: str
    portSpecification:
      description:
      - 'Specifies how port is selected for health checking, can be one of the following
        values: * `USE_FIXED_PORT`: The port number in `port` is used for health checking.'
      - "* `USE_NAMED_PORT`: The `portName` is used for health checking."
      - "* `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the port
        or named port specified in the Backend Service is used for health checking."
      - If not specified, TCP health check follows behavior specified in `port` and
        `portName` fields.
      returned: success
      type: str
sslHealthCheck:
  description:
  - A nested object resource.
  returned: success
  type: complex
  contains:
    request:
      description:
      - The application data to send once the SSL connection has been established
        (default value is empty). If both request and response are empty, the connection
        establishment alone will indicate health. The request data can only be ASCII.
      returned: success
      type: str
    response:
      description:
      - The bytes to match against the beginning of the response data. If left empty
        (the default value), any response will indicate health. The response data
        can only be ASCII.
      returned: success
      type: str
    port:
      description:
      - The TCP port number for the SSL health check request.
      - The default value is 443.
      returned: success
      type: int
    portName:
      description:
      - Port name as defined in InstanceGroup#NamedPort#name. If both port and port_name
        are defined, port takes precedence.
      returned: success
      type: str
    proxyHeader:
      description:
      - Specifies the type of proxy header to append before sending data to the backend,
        either NONE or PROXY_V1. The default is NONE.
      returned: success
      type: str
    portSpecification:
      description:
      - 'Specifies how port is selected for health checking, can be one of the following
        values: * `USE_FIXED_PORT`: The port number in `port` is used for health checking.'
      - "* `USE_NAMED_PORT`: The `portName` is used for health checking."
      - "* `USE_SERVING_PORT`: For NetworkEndpointGroup, the port specified for each
        network endpoint is used for health checking. For other backends, the port
        or named port specified in the Backend Service is used for health checking."
      - If not specified, SSL health check follows behavior specified in `port` and
        `portName` fields.
      returned: success
      type: str
'''

################################################################################
# Imports
################################################################################

from ansible.module_utils.gcp_utils import navigate_hash, GcpSession, GcpModule, GcpRequest, remove_nones_from_dict, replace_resource_dict
import json
import time

################################################################################
# Main
################################################################################


def main():
    """Main function"""

    module = GcpModule(
        argument_spec=dict(
            state=dict(default='present', choices=['present', 'absent'], type='str'),
            check_interval_sec=dict(default=5, type='int'),
            description=dict(type='str'),
            healthy_threshold=dict(default=2, type='int'),
            name=dict(required=True, type='str'),
            timeout_sec=dict(default=5, type='int', aliases=['timeout_seconds']),
            unhealthy_threshold=dict(default=2, type='int'),
            type=dict(type='str'),
            http_health_check=dict(
                type='dict',
                options=dict(
                    host=dict(type='str'),
                    request_path=dict(default='/', type='str'),
                    response=dict(type='str'),
                    port=dict(type='int'),
                    port_name=dict(type='str'),
                    proxy_header=dict(default='NONE', type='str'),
                    port_specification=dict(type='str'),
                ),
            ),
            https_health_check=dict(
                type='dict',
                options=dict(
                    host=dict(type='str'),
                    request_path=dict(default='/', type='str'),
                    response=dict(type='str'),
                    port=dict(type='int'),
                    port_name=dict(type='str'),
                    proxy_header=dict(default='NONE', type='str'),
                    port_specification=dict(type='str'),
                ),
            ),
            tcp_health_check=dict(
                type='dict',
                options=dict(
                    request=dict(type='str'),
                    response=dict(type='str'),
                    port=dict(type='int'),
                    port_name=dict(type='str'),
                    proxy_header=dict(default='NONE', type='str'),
                    port_specification=dict(type='str'),
                ),
            ),
            ssl_health_check=dict(
                type='dict',
                options=dict(
                    request=dict(type='str'),
                    response=dict(type='str'),
                    port=dict(type='int'),
                    port_name=dict(type='str'),
                    proxy_header=dict(default='NONE', type='str'),
                    port_specification=dict(type='str'),
                ),
            ),
        ),
        mutually_exclusive=[['http_health_check', 'https_health_check', 'ssl_health_check', 'tcp_health_check']],
    )

    if not module.params['scopes']:
        module.params['scopes'] = ['https://www.googleapis.com/auth/compute']

    state = module.params['state']
    kind = 'compute#healthCheck'

    fetch = fetch_resource(module, self_link(module), kind)
    changed = False

    if fetch:
        if state == 'present':
            if is_different(module, fetch):
                update(module, self_link(module), kind)
                fetch = fetch_resource(module, self_link(module), kind)
                changed = True
        else:
            delete(module, self_link(module), kind)
            fetch = {}
            changed = True
    else:
        if state == 'present':
            fetch = create(module, collection(module), kind)
            changed = True
        else:
            fetch = {}

    fetch.update({'changed': changed})

    module.exit_json(**fetch)


def create(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.post(link, resource_to_request(module)))


def update(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.put(link, resource_to_request(module)))


def delete(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.delete(link))


def resource_to_request(module):
    request = {
        u'kind': 'compute#healthCheck',
        u'checkIntervalSec': module.params.get('check_interval_sec'),
        u'description': module.params.get('description'),
        u'healthyThreshold': module.params.get('healthy_threshold'),
        u'name': module.params.get('name'),
        u'timeoutSec': module.params.get('timeout_sec'),
        u'unhealthyThreshold': module.params.get('unhealthy_threshold'),
        u'type': module.params.get('type'),
        u'httpHealthCheck': HealthCheckHttphealthcheck(module.params.get('http_health_check', {}), module).to_request(),
        u'httpsHealthCheck': HealthCheckHttpshealthcheck(module.params.get('https_health_check', {}), module).to_request(),
        u'tcpHealthCheck': HealthCheckTcphealthcheck(module.params.get('tcp_health_check', {}), module).to_request(),
        u'sslHealthCheck': HealthCheckSslhealthcheck(module.params.get('ssl_health_check', {}), module).to_request(),
    }
    return_vals = {}
    for k, v in request.items():
        if v or v is False:
            return_vals[k] = v

    return return_vals


def fetch_resource(module, link, kind, allow_not_found=True):
    auth = GcpSession(module, 'compute')
    return return_if_object(module, auth.get(link), kind, allow_not_found)


def self_link(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/global/healthChecks/{name}".format(**module.params)


def collection(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/global/healthChecks".format(**module.params)


def return_if_object(module, response, kind, allow_not_found=False):
    # If not found, return nothing.
    if allow_not_found and response.status_code == 404:
        return None

    # If no content, return nothing.
    if response.status_code == 204:
        return None

    try:
        module.raise_for_status(response)
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError):
        module.fail_json(msg="Invalid JSON response with error: %s" % response.text)

    if navigate_hash(result, ['error', 'errors']):
        module.fail_json(msg=navigate_hash(result, ['error', 'errors']))

    return result


def is_different(module, response):
    request = resource_to_request(module)
    response = response_to_hash(module, response)

    # Remove all output-only from response.
    response_vals = {}
    for k, v in response.items():
        if k in request:
            response_vals[k] = v

    request_vals = {}
    for k, v in request.items():
        if k in response:
            request_vals[k] = v

    return GcpRequest(request_vals) != GcpRequest(response_vals)


# Remove unnecessary properties from the response.
# This is for doing comparisons with Ansible's current parameters.
def response_to_hash(module, response):
    return {
        u'checkIntervalSec': response.get(u'checkIntervalSec'),
        u'creationTimestamp': response.get(u'creationTimestamp'),
        u'description': response.get(u'description'),
        u'healthyThreshold': response.get(u'healthyThreshold'),
        u'id': response.get(u'id'),
        u'name': module.params.get('name'),
        u'timeoutSec': response.get(u'timeoutSec'),
        u'unhealthyThreshold': response.get(u'unhealthyThreshold'),
        u'type': response.get(u'type'),
        u'httpHealthCheck': HealthCheckHttphealthcheck(response.get(u'httpHealthCheck', {}), module).from_response(),
        u'httpsHealthCheck': HealthCheckHttpshealthcheck(response.get(u'httpsHealthCheck', {}), module).from_response(),
        u'tcpHealthCheck': HealthCheckTcphealthcheck(response.get(u'tcpHealthCheck', {}), module).from_response(),
        u'sslHealthCheck': HealthCheckSslhealthcheck(response.get(u'sslHealthCheck', {}), module).from_response(),
    }


def async_op_url(module, extra_data=None):
    if extra_data is None:
        extra_data = {}
    url = "https://www.googleapis.com/compute/v1/projects/{project}/global/operations/{op_id}"
    combined = extra_data.copy()
    combined.update(module.params)
    return url.format(**combined)


def wait_for_operation(module, response):
    op_result = return_if_object(module, response, 'compute#operation')
    if op_result is None:
        return {}
    status = navigate_hash(op_result, ['status'])
    wait_done = wait_for_completion(status, op_result, module)
    return fetch_resource(module, navigate_hash(wait_done, ['targetLink']), 'compute#healthCheck')


def wait_for_completion(status, op_result, module):
    op_id = navigate_hash(op_result, ['name'])
    op_uri = async_op_url(module, {'op_id': op_id})
    while status != 'DONE':
        raise_if_errors(op_result, ['error', 'errors'], module)
        time.sleep(1.0)
        op_result = fetch_resource(module, op_uri, 'compute#operation', False)
        status = navigate_hash(op_result, ['status'])
    return op_result


def raise_if_errors(response, err_path, module):
    errors = navigate_hash(response, err_path)
    if errors is not None:
        module.fail_json(msg=errors)


class HealthCheckHttphealthcheck(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {
                u'host': self.request.get('host'),
                u'requestPath': self.request.get('request_path'),
                u'response': self.request.get('response'),
                u'port': self.request.get('port'),
                u'portName': self.request.get('port_name'),
                u'proxyHeader': self.request.get('proxy_header'),
                u'portSpecification': self.request.get('port_specification'),
            }
        )

    def from_response(self):
        return remove_nones_from_dict(
            {
                u'host': self.request.get(u'host'),
                u'requestPath': self.request.get(u'requestPath'),
                u'response': self.request.get(u'response'),
                u'port': self.request.get(u'port'),
                u'portName': self.request.get(u'portName'),
                u'proxyHeader': self.request.get(u'proxyHeader'),
                u'portSpecification': self.request.get(u'portSpecification'),
            }
        )


class HealthCheckHttpshealthcheck(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {
                u'host': self.request.get('host'),
                u'requestPath': self.request.get('request_path'),
                u'response': self.request.get('response'),
                u'port': self.request.get('port'),
                u'portName': self.request.get('port_name'),
                u'proxyHeader': self.request.get('proxy_header'),
                u'portSpecification': self.request.get('port_specification'),
            }
        )

    def from_response(self):
        return remove_nones_from_dict(
            {
                u'host': self.request.get(u'host'),
                u'requestPath': self.request.get(u'requestPath'),
                u'response': self.request.get(u'response'),
                u'port': self.request.get(u'port'),
                u'portName': self.request.get(u'portName'),
                u'proxyHeader': self.request.get(u'proxyHeader'),
                u'portSpecification': self.request.get(u'portSpecification'),
            }
        )


class HealthCheckTcphealthcheck(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {
                u'request': self.request.get('request'),
                u'response': self.request.get('response'),
                u'port': self.request.get('port'),
                u'portName': self.request.get('port_name'),
                u'proxyHeader': self.request.get('proxy_header'),
                u'portSpecification': self.request.get('port_specification'),
            }
        )

    def from_response(self):
        return remove_nones_from_dict(
            {
                u'request': self.request.get(u'request'),
                u'response': self.request.get(u'response'),
                u'port': self.request.get(u'port'),
                u'portName': self.request.get(u'portName'),
                u'proxyHeader': self.request.get(u'proxyHeader'),
                u'portSpecification': self.request.get(u'portSpecification'),
            }
        )


class HealthCheckSslhealthcheck(object):
    def __init__(self, request, module):
        self.module = module
        if request:
            self.request = request
        else:
            self.request = {}

    def to_request(self):
        return remove_nones_from_dict(
            {
                u'request': self.request.get('request'),
                u'response': self.request.get('response'),
                u'port': self.request.get('port'),
                u'portName': self.request.get('port_name'),
                u'proxyHeader': self.request.get('proxy_header'),
                u'portSpecification': self.request.get('port_specification'),
            }
        )

    def from_response(self):
        return remove_nones_from_dict(
            {
                u'request': self.request.get(u'request'),
                u'response': self.request.get(u'response'),
                u'port': self.request.get(u'port'),
                u'portName': self.request.get(u'portName'),
                u'proxyHeader': self.request.get(u'proxyHeader'),
                u'portSpecification': self.request.get(u'portSpecification'),
            }
        )


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2019, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_firewall_log_profile_network
short_description: Configures Network Firewall related settings of the log profile
description:
  - Configures Network Firewall related settings of the log profile.
version_added: 2.9
options:
  profile_name:
    description:
      - Specifies the name of the AFM log profile to be updated.
    type: str
    required: True
  log_publisher:
    description:
      - Specifies the name of the log publisher used for Network events.
      - To specify the log_publisher on a different partition from the AFM log profile, specify the name in fullpath
        format, e.g. C(/Foobar/log-publisher), otherwise the partition for log publisher is inferred from C(partition)
        module parameter.
    type: str
  rate_limit:
    description:
      - Defines a rate limit for all combined network firewall log messages per second. Beyond this rate limit,
        log messages are not logged.
      - To specify an indefinite rate, use the value C(indefinite).
      - If specifying a numeric rate, the value must be between C(1) and C(4294967295).
    type: str
  log_matches_accept_rule:
    description:
      - Modify log settings for ACL rules configured with an "accept" or "accept decisively" action.
    suboptions:
      enabled:
        description:
          - This option is used to enable or disable the logging of packets that match ACL rules configured with
            an "accept" or "accept decisively" action.
        type: bool
      rate_limit:
        description:
          - This option is used to set rate limits for the logging of packets that match ACL rules
            configured with an "accept" or "accept decisively" action.
          - This option is effective only if logging of this message type is enabled.
        type: int
    type: dict
  log_matches_drop_rule:
    description:
      - Modify log settings for ACL rules configured with a drop action.
    suboptions:
      enabled:
        description:
          - This option is used to enable or disable the logging of packets that match ACL rules
            configured with a drop action.
        type: bool
      rate_limit:
        description:
          - This option is used to set rate limits for the logging of packets that match ACL rules
            configured with a drop action.
          - This option is effective only if logging of this message type is enabled.
        type: int
    type: dict
  log_matches_reject_rule:
    description:
      - Modify log settings for ACL rules configured with a reject action.
    suboptions:
      enabled:
        description:
          - This option is used to enable or disable the logging of packets that match ACL rules
            configured with a reject action.
        type: bool
      rate_limit:
        description:
          - This option is used to set rate limits for the logging of packets that match ACL rules
            configured with a reject action.
          - This option is effective only if logging of this message type is enabled.
        type: int
    type: dict
  log_ip_errors:
    description:
      - Modify log settings for logging of IP error packets.
    suboptions:
      enabled:
        description:
          - This option is used to enable or disable the logging of IP error packets.
        type: bool
      rate_limit:
        description:
          - This option is used to set rate limits for the logging of IP error packets.
          - This option is effective only if logging of this message type is enabled.
        type: int
    type: dict
  log_tcp_errors:
    description:
      - Modify log settings for logging of TCP error packets.
    suboptions:
      enabled:
        description:
          - This option is used to enable or disable the logging of TCP error packets.
        type: bool
      rate_limit:
        description:
          - This option is used to set rate limits for the logging of TCP error packets.
          - This option is effective only if logging of this message type is enabled.
        type: int
    type: dict
  log_tcp_events:
    description:
      - Modify log settings for logging of TCP events on the client side.
    suboptions:
      enabled:
        description:
          - This option is used to enable or disable the logging of TCP events on the client side.
          - Only 'Established' and 'Closed' states of a TCP session are logged if this option is enabled.
        type: bool
      rate_limit:
        description:
          - This option is used to set rate limits for the logging of TCP events on the client side.
          - This option is effective only if logging of this message type is enabled.
        type: int
    type: dict
  log_translation_fields:
    description:
      - This option is used to enable or disable the logging of translated (i.e server side) fields in ACL
        match and TCP events.
      - Translated fields include (but are not limited to) source address/port, destination address/port,
        IP protocol, route domain, and VLAN.
    type: bool
  log_storage_format:
    description:
      - Specifies the type of the storage format.
      - When creating a new log profile, if this parameter is not specified, the default is C(none).
      - When C(field-list), specifies that the log displays only the items you specify in the C(log_message_fields) list
        with C(log_format_delimiter) as the delimiter between the items.
      - When C(none), the messages will be logged in the default format, which is C("management_ip_address",
        "bigip_hostname","context_type", "context_name","src_geo","src_ip", "dest_geo","dest_ip","src_port",
        "dest_port","vlan","protocol","route_domain", "translated_src_ip", "translated_dest_ip",
        "translated_src_port","translated_dest_port", "translated_vlan","translated_ip_protocol",
        "translated_route_domain", "acl_policy_type", "acl_policy_name","acl_rule_name","action",
        "drop_reason","sa_translation_type", "sa_translation_pool","flow_id", "source_user",
        "source_fqdn","dest_fqdn").
    choices:
      - field-list
      - none
    type: str
  log_format_delimiter:
    description:
      - Specifies the delimiter string when using a C(log_storage_format) of C(field-list).
      - When creating a new profile, if this parameter is not specified, the default value of C(,)
        (the comma character) will be used.
      - This option is valid when the C(log_storage_format) is set to C(field-list). It will be ignored otherwise.
      - Depending on the delimiter used, it may be necessary to wrap the delimiter
        in quotes to prevent YAML errors from occurring.
      - The special character C($) should not be used, and will raise an error if used,
        as it is reserved for internal use.
      - The maximum length allowed for this parameter is C(31) characters.
    type: str
  log_message_fields:
    description:
      - Specifies a set of fields to be logged.
      - This option is valid when the C(log_storage_format) is set to C(field-list). It will be ignored otherwise.
      - The order of the list is important as the server displays the selected traffic items in the log
        sequentially according to it.
    type: list
    choices:
      - acl_policy_name
      - acl_policy_type
      - acl_rule_name
      - action
      - bigip_hostname
      - context_name
      - context_type
      - date_time
      - dest_fqdn
      - dest_geo
      - dest_ip
      - dest_port
      - drop_reason
      - management_ip_address
      - protocol
      - route_domain
      - sa_translation_pool
      - sa_translation_type
      - source_fqdn
      - source_user
      - src_geo
      - src_ip
      - src_port
      - translated_dest_ip
      - translated_dest_port
      - translated_ip_protocol
      - translated_route_domain
      - translated_src_ip
      - translated_src_port
      - translated_vlan
      - vlan
  partition:
    description:
      - Device partition to create log profile on.
      - Parameter also used when specifying names for log publishers, unless log publisher names are in fullpath format.
    type: str
    default: Common
  state:
    description:
      - When C(state) is C(present), ensures the resource exists.
      - Only built-in profile that allows updating network log settings is global-network, attempts to do so on other
        built-in profiles will be ignored.
      - When C(state) is C(absent), ensures that resource is removed.
      - The C(absent) state is ignored for global-network log profile.
    type: str
    choices:
      - present
      - absent
    default: present
extends_documentation_fragment: f5
author:
  - Wojciech Wypior (@wojtek0806)
'''

EXAMPLES = r'''
- name: Add network settings to log profile
  bigip_firewall_log_profile_network:
    profile_name: barbaz
    rate_limit: 150000
    log_publisher: local-db-pub
    log_tcp_errors:
      enabled: yes
      rate_limit: 10000
    log_tcp_events:
      enabled: yes
      rate_limit: 40000
    log_storage_format: "field-list"
    log_message_fields:
      - vlan
      - translated_vlan
      - src_ip
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost

- name: Change delimiter and log fields
  bigip_firewall_log_profile_network:
    profile_name: barbaz
    log_format_delimiter: '.'
    log_message_fields:
      - translated_dest_ip
      - translated_dest_port
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost

- name: Modify built-in profile
  bigip_firewall_log_profile_network:
    profile_name: "global-network"
    log_publisher: "/foobar/log1"
    log_ip_errors:
      enabled: yes
      rate_limit: 60000
    log_matches_reject_rule:
      enabled: yes
      rate_limit: 2000
    log_translation_fields: yes
    log_storage_format: "field-list"
    log_format_delimiter: '.'
    log_message_fields:
      - protocol
      - dest_ip
      - dest_port
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost

- name: Remove custom log profile network log settings
  bigip_firewall_log_profile_network:
    profile_name: "{{ log_profile }}"
    state: absent
    provider:
      password: secret
      server: lb.mydomain.com
      user: admin
  delegate_to: localhost
'''

RETURN = r'''
log_publisher:
  description: The name of the log publisher used for Network events.
  returned: changed
  type: str
  sample: /Common/log-publisher
rate_limit:
  description: The rate limit for all combined network firewall log messages per second.
  returned: changed
  type: str
  sample: "indefinite"
log_matches_accept_rule:
  description: Log settings for ACL rules configured with an "accept" or "accept decisively" action.
  type: complex
  returned: changed
  contains:
    enabled:
      description: Enable or disable the logging of packets that match ACL rules.
      returned: changed
      type: bool
      sample: yes
    rate_limit:
      description: The rate limit for the logging of packets that match ACL rules.
      returned: changed
      type: str
      sample: "indefinite"
  sample: hash/dictionary of values
log_matches_drop_rule:
  description: Log settings for ACL rules configured with a drop action.
  type: complex
  returned: changed
  contains:
    enabled:
      description: Enable or disable the logging of packets that match ACL rules.
      returned: changed
      type: bool
      sample: yes
    rate_limit:
      description: The rate limit for the logging of packets that match ACL rules.
      returned: changed
      type: str
      sample: "indefinite"
  sample: hash/dictionary of values
log_matches_reject_rule:
  description: Log settings for ACL rules configured with a reject action.
  type: complex
  returned: changed
  contains:
    enabled:
      description: Enable or disable the logging of packets that match ACL rules.
      returned: changed
      type: bool
      sample: yes
    rate_limit:
      description: The rate limit for the logging of packets that match ACL rules.
      returned: changed
      type: str
      sample: "indefinite"
  sample: hash/dictionary of values
log_ip_errors:
  description: Log settings for logging of IP error packets.
  type: complex
  returned: changed
  contains:
    enabled:
      description: Enable or disable the logging of IP error packets.
      returned: changed
      type: bool
      sample: yes
    rate_limit:
      description: The rate limit for the logging of IP error packets.
      returned: changed
      type: str
      sample: "indefinite"
  sample: hash/dictionary of values
log_tcp_errors:
  description: Log settings for logging of TCP error packets.
  type: complex
  returned: changed
  contains:
    enabled:
      description: Enable or disable the logging of TCP error packets.
      returned: changed
      type: bool
      sample: yes
    rate_limit:
      description: The rate limit for the logging of TCP error packets.
      returned: changed
      type: str
      sample: "indefinite"
  sample: hash/dictionary of values
log_tcp_events:
  description: Log settings for logging of TCP events on the client side.
  type: complex
  returned: changed
  contains:
    enabled:
      description: Enable or disable the logging of TCP events on the client side.
      returned: changed
      type: bool
      sample: yes
    rate_limit:
      description: The rate limit for the logging of TCP events on the client side.
      returned: changed
      type: str
      sample: "indefinite"
  sample: hash/dictionary of values
log_translation_fields:
  description: Enable or disable the logging of translated (i.e server side) fields in ACL match and TCP events.
  returned: changed
  type: bool
  sample: yes
log_storage_format:
  description: The type of the storage format.
  returned: changed
  type: str
  sample: "field-list"
log_format_delimiter:
  description: The delimiter string when using a log_storage_format of field-list.
  returned: changed
  type: str
  sample: "."
log_message_fields:
  description: The delimiter string when using a log_storage_format of field-list.
  returned: changed
  type: list
  sample: ["acl_policy_name", "acl_policy_type"]
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.basic import env_fallback

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import fq_name
    from library.module_utils.network.f5.common import transform_name
    from library.module_utils.network.f5.common import f5_argument_spec
    from library.module_utils.network.f5.common import flatten_boolean
    from library.module_utils.network.f5.compare import cmp_str_with_none
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import fq_name
    from ansible.module_utils.network.f5.common import transform_name
    from ansible.module_utils.network.f5.common import f5_argument_spec
    from ansible.module_utils.network.f5.common import flatten_boolean
    from ansible.module_utils.network.f5.compare import cmp_str_with_none


class Parameters(AnsibleF5Parameters):
    api_map = {
        'publisher': 'log_publisher',
        'rateLimit': 'rate_limits',
    }

    api_attributes = [
        'publisher',
        'format',
        'rateLimit',
        'filter',
    ]

    returnables = [
        'rate_acl_match_accept',
        'rate_acl_match_drop',
        'rate_acl_match_reject',
        'rate_tcp_errors',
        'rate_tcp_events',
        'rate_ip_errors',
        'rate_limit',
        'log_acl_match_accept',
        'log_acl_match_drop',
        'log_acl_match_reject',
        'log_tcp_errors',
        'log_tcp_events',
        'log_ip_errors',
        'log_translation_fields',
        'log_publisher',
        'log_format_delimiter',
        'log_storage_format',
        'log_message_fields',
        'log_matches_accept_rule',
        'log_matches_drop_rule',
        'log_matches_reject_rule',
    ]

    updatables = [
        'rate_acl_match_accept',
        'rate_acl_match_drop',
        'rate_acl_match_reject',
        'rate_tcp_errors',
        'rate_tcp_events',
        'rate_ip_errors',
        'rate_limit',
        'log_acl_match_accept',
        'log_acl_match_drop',
        'log_acl_match_reject',
        'log_tcp_errors',
        'log_tcp_events',
        'log_ip_errors',
        'log_translation_fields',
        'log_publisher',
        'log_format_delimiter',
        'log_storage_format',
        'log_message_fields',
    ]


class ApiParameters(Parameters):
    @property
    def rate_acl_match_accept(self):
        if self._values['rate_limits'] is None:
            return None
        return self._values['rate_limits']['aclMatchAccept']

    @property
    def log_acl_match_accept(self):
        if self._values['filter'] is None:
            return None
        return self._values['filter']['logAclMatchAccept']

    @property
    def rate_acl_match_drop(self):
        if self._values['rate_limits'] is None:
            return None
        return self._values['rate_limits']['aclMatchDrop']

    @property
    def log_acl_match_drop(self):
        if self._values['filter'] is None:
            return None
        return self._values['filter']['logAclMatchDrop']

    @property
    def rate_acl_match_reject(self):
        if self._values['rate_limits'] is None:
            return None
        return self._values['rate_limits']['aclMatchReject']

    @property
    def log_acl_match_reject(self):
        if self._values['filter'] is None:
            return None
        return self._values['filter']['logAclMatchReject']

    @property
    def rate_tcp_errors(self):
        if self._values['rate_limits'] is None:
            return None
        return self._values['rate_limits']['tcpErrors']

    @property
    def log_tcp_errors(self):
        if self._values['filter'] is None:
            return None
        return self._values['filter']['logTcpErrors']

    @property
    def rate_tcp_events(self):
        if self._values['rate_limits'] is None:
            return None
        return self._values['rate_limits']['tcpEvents']

    @property
    def log_tcp_events(self):
        if self._values['filter'] is None:
            return None
        return self._values['filter']['logTcpEvents']

    @property
    def rate_ip_errors(self):
        if self._values['rate_limits'] is None:
            return None
        return self._values['rate_limits']['ipErrors']

    @property
    def log_ip_errors(self):
        if self._values['filter'] is None:
            return None
        return self._values['filter']['logIpErrors']

    @property
    def log_translation_fields(self):
        if self._values['filter'] is None:
            return None
        return self._values['filter']['logTranslationFields']

    @property
    def rate_limit(self):
        if self._values['rate_limits'] is None:
            return None
        return self._values['rate_limits']['aggregateRate']

    @property
    def log_format_delimiter(self):
        if self._values['format'] is None:
            return None
        return self._values['format']['fieldListDelimiter']

    @property
    def log_storage_format(self):
        if self._values['format'] is None:
            return None
        return self._values['format']['type']

    @property
    def log_message_fields(self):
        if self._values['format'] is None:
            return None
        if 'fieldList' in self._values['format']:
            return self._values['format']['fieldList']


class ModuleParameters(Parameters):
    def _validate_aggregate_rate(self, aggregate_rate):
        if aggregate_rate is None:
            return None
        if aggregate_rate == 'indefinite':
            return 4294967295
        if 0 <= int(aggregate_rate) <= 4294967295:
            return int(aggregate_rate)
        raise F5ModuleError(
            "Valid 'maximum_age' must be in range 0 - 4294967295, or 'indefinite'."
        )

    @property
    def rate_acl_match_accept(self):
        if self._values['log_matches_accept_rule'] is None:
            return None
        return self._validate_aggregate_rate(self._values['log_matches_accept_rule']['rate_limit'])

    @property
    def log_acl_match_accept(self):
        if self._values['log_matches_accept_rule'] is None:
            return None
        result = flatten_boolean(self._values['log_matches_accept_rule']['enabled'])
        if result == 'yes':
            return 'enabled'
        if result == 'no':
            return 'disabled'
        return result

    @property
    def rate_acl_match_drop(self):
        if self._values['log_matches_drop_rule'] is None:
            return None
        return self._validate_aggregate_rate(self._values['log_matches_drop_rule']['rate_limit'])

    @property
    def log_acl_match_drop(self):
        if self._values['log_matches_drop_rule'] is None:
            return None
        result = flatten_boolean(self._values['log_matches_drop_rule']['enabled'])
        if result == 'yes':
            return 'enabled'
        if result == 'no':
            return 'disabled'
        return result

    @property
    def rate_acl_match_reject(self):
        if self._values['log_matches_reject_rule'] is None:
            return None
        return self._validate_aggregate_rate(self._values['log_matches_reject_rule']['rate_limit'])

    @property
    def log_acl_match_reject(self):
        if self._values['log_matches_reject_rule'] is None:
            return None
        result = flatten_boolean(self._values['log_matches_reject_rule']['enabled'])
        if result == 'yes':
            return 'enabled'
        if result == 'no':
            return 'disabled'
        return result

    @property
    def rate_tcp_errors(self):
        if self._values['log_tcp_errors'] is None:
            return None
        return self._validate_aggregate_rate(self._values['log_tcp_errors']['rate_limit'])

    @property
    def log_tcp_errors(self):
        if self._values['log_tcp_errors'] is None:
            return None
        result = flatten_boolean(self._values['log_tcp_errors']['enabled'])
        if result == 'yes':
            return 'enabled'
        if result == 'no':
            return 'disabled'
        return result

    @property
    def rate_tcp_events(self):
        if self._values['log_tcp_events'] is None:
            return None
        return self._validate_aggregate_rate(self._values['log_tcp_events']['rate_limit'])

    @property
    def log_tcp_events(self):
        if self._values['log_tcp_events'] is None:
            return None
        result = flatten_boolean(self._values['log_tcp_events']['enabled'])
        if result == 'yes':
            return 'enabled'
        if result == 'no':
            return 'disabled'
        return result

    @property
    def rate_ip_errors(self):
        if self._values['log_ip_errors'] is None:
            return None
        return self._validate_aggregate_rate(self._values['log_ip_errors']['rate_limit'])

    @property
    def log_ip_errors(self):
        if self._values['log_ip_errors'] is None:
            return None
        result = flatten_boolean(self._values['log_ip_errors']['enabled'])
        if result == 'yes':
            return 'enabled'
        if result == 'no':
            return 'disabled'
        return result

    @property
    def log_translation_fields(self):
        result = flatten_boolean(self._values['log_translation_fields'])
        if result == 'yes':
            return 'enabled'
        if result == 'no':
            return 'disabled'
        return result

    @property
    def log_publisher(self):
        log_publisher = self._values['log_publisher']
        if log_publisher is None:
            return None
        if log_publisher in ['', 'none']:
            return log_publisher
        return fq_name(self.partition, log_publisher)

    @property
    def rate_limit(self):
        return self._validate_aggregate_rate(self._values['rate_limit'])

    @property
    def log_format_delimiter(self):
        if self._values['log_format_delimiter'] is None:
            return None
        if len(self._values['log_format_delimiter']) > 31:
            raise F5ModuleError('The maximum length of delimiter is 31 characters.')
        if "$" in self._values['log_format_delimiter']:
            raise F5ModuleError("Cannot use '$' character as a part of delimiter.")
        return self._values['log_format_delimiter']


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                result[returnable] = getattr(self, returnable)
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    @property
    def rate_limits(self):
        to_filter = dict(
            aclMatchAccept=self._values['rate_acl_match_accept'],
            aclMatchDrop=self._values['rate_acl_match_drop'],
            aclMatchReject=self._values['rate_acl_match_reject'],
            ipErrors=self._values['rate_ip_errors'],
            tcpErrors=self._values['rate_tcp_errors'],
            tcpEvents=self._values['rate_tcp_events'],
            aggregateRate=self._values['rate_limit'],
        )
        result = self._filter_params(to_filter)
        return result

    @property
    def format(self):
        to_filter = dict(
            fieldListDelimiter=self._values['log_format_delimiter'],
            type=self._values['log_storage_format'],
            fieldList=self._values['log_message_fields']
        )
        result = self._filter_params(to_filter)
        return result

    @property
    def filter(self):
        to_filter = dict(
            logAclMatchAccept=self._values['log_acl_match_accept'],
            logAclMatchDrop=self._values['log_acl_match_drop'],
            logAclMatchReject=self._values['log_acl_match_reject'],
            logIpErrors=self._values['log_ip_errors'],
            logTcpErrors=self._values['log_tcp_errors'],
            logTcpEvents=self._values['log_tcp_events'],
            logTranslationFields=self._values['log_translation_fields'],
        )
        result = self._filter_params(to_filter)
        return result


class ReportableChanges(Changes):
    def _change_aggregate_rate_value(self, value):
        if value == 4294967295:
            return 'indefinite'
        else:
            return value

    def _rebuild_params(self, enabled, rate_limit):
        to_filter = dict(
            enabled=flatten_boolean(self._values[enabled]),
            rate_limit=self._change_aggregate_rate_value(self._values[rate_limit])
        )
        result = self._filter_params(to_filter)
        return result

    @property
    def log_matches_accept_rule(self):
        result = self._rebuild_params('log_acl_match_accept', 'rate_acl_match_accept')
        if result:
            return result

    @property
    def log_acl_match_accept(self):
        return None

    @property
    def rate_acl_match_accept(self):
        return None

    @property
    def log_matches_drop_rule(self):
        result = self._rebuild_params('log_acl_match_drop', 'rate_acl_match_drop')
        if result:
            return result

    @property
    def log_acl_match_drop(self):
        return None

    @property
    def rate_acl_match_drop(self):
        return None

    @property
    def log_matches_reject_rule(self):
        result = self._rebuild_params('log_acl_match_reject', 'rate_acl_match_reject')
        if result:
            return result

    @property
    def log_acl_match_reject(self):
        return None

    @property
    def rate_acl_match_reject(self):
        return None

    @property
    def log_ip_errors(self):
        result = self._rebuild_params('log_ip_errors', 'rate_ip_errors')
        if result:
            return result

    @property
    def rate_ip_errors(self):
        return None

    @property
    def log_tcp_errors(self):
        result = self._rebuild_params('log_tcp_errors', 'rate_tcp_errors')
        if result:
            return result

    @property
    def rate_tcp_errors(self):
        return None

    @property
    def log_tcp_events(self):
        result = self._rebuild_params('log_tcp_events', 'rate_tcp_events')
        if result:
            return result

    @property
    def rate_tcp_events(self):
        return None


class Difference(object):
    def __init__(self, want, have=None):
        self.want = want
        self.have = have

    def compare(self, param):
        try:
            result = getattr(self, param)
            return result
        except AttributeError:
            return self.__default(param)

    def __default(self, param):
        attr1 = getattr(self.want, param)
        try:
            attr2 = getattr(self.have, param)
            if attr1 != attr2:
                return attr1
        except AttributeError:
            return attr1

    @property
    def log_message_fields(self):
        if self.want.log_message_fields is None:
            return None
        if len(self.want.log_message_fields) == 1:
            if self.have.log_message_fields is None and self.want.log_message_fields[0] in ['', 'none']:
                return None
            if self.have.log_message_fields is not None and self.want.log_message_fields[0] in ['', 'none']:
                return []
        if self.have.log_message_fields is None:
            return self.want.log_message_fields
        if set(self.want.log_message_fields) != set(self.have.log_message_fields):
            return self.want.log_message_fields
        return None

    @property
    def log_publisher(self):
        return cmp_str_with_none(self.want.log_publisher, self.have.log_publisher)


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.get('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params)
        self.have = ApiParameters()
        self.changes = UsableChanges()

    def _set_changed_options(self):
        changed = {}
        for key in Parameters.returnables:
            if getattr(self.want, key) is not None:
                changed[key] = getattr(self.want, key)
        if changed:
            self.changes = UsableChanges(params=changed)

    def _update_changed_options(self):
        diff = Difference(self.want, self.have)
        updatables = Parameters.updatables
        changed = dict()
        for k in updatables:
            change = diff.compare(k)
            if change is None:
                continue
            else:
                if isinstance(change, dict):
                    changed.update(change)
                else:
                    changed[k] = change
        if changed:
            self.changes = UsableChanges(params=changed)
            return True
        return False

    def _announce_deprecations(self, result):
        warnings = result.pop('__warnings', [])
        for warning in warnings:
            self.client.module.deprecate(
                msg=warning['msg'],
                version=warning['version']
            )

    def exec_module(self):
        changed = False
        result = dict()
        state = self.want.state

        if state == "present":
            changed = self.present()
        elif state == "absent":
            changed = self.absent()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=changed))
        self._announce_deprecations(result)
        return result

    def present(self):
        built_ins = ['Log all requests', 'Log illegal requests', 'local-dos']
        if self.want.profile_name in built_ins:
            return False
        if self.exists():
            return self.update()
        else:
            return self.create()

    def absent(self):
        # Built-in profile global-network cannot disable network log profile
        if 'global-network' in self.want.profile_name:
            return False
        if self.exists():
            return self.remove()
        return False

    def should_update(self):
        result = self._update_changed_options()
        if result:
            return True
        return False

    def update(self):
        self.have = self.read_current_from_device()
        if not self.should_update():
            return False
        if self.module.check_mode:
            return True
        self.update_on_device()
        return True

    def remove(self):
        if self.module.check_mode:
            return True
        self.remove_from_device()
        if self.exists():
            raise F5ModuleError("Failed to delete the resource.")
        return True

    def create(self):
        self._set_changed_options()
        if self.module.check_mode:
            return True
        self.create_on_device()
        return True

    def _internal_name(self):
        name = self.want.profile_name
        partition = self.want.partition
        if 'global-network' in name:
            return 'global-network'
        return transform_name(partition, name)

    def _profile_exists(self):
        uri = "https://{0}:{1}/mgmt/tm/security/log/profile/{2}/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.profile_name),
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError:
            return False
        if resp.status == 404 or 'code' in response and response['code'] == 404:
            return False
        return True

    def exists(self):
        if not self._profile_exists():
            raise F5ModuleError(
                "Specified AFM log profile: {0} does not exist".format(self.want.profile_name)
            )
        uri = "https://{0}:{1}/mgmt/tm/security/log/profile/{2}/network/{3}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.profile_name),
            self._internal_name()
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError:
            return False
        if resp.status == 404 or 'code' in response and response['code'] == 404:
            return False
        return True

    def create_on_device(self):
        params = self.changes.api_params()
        params['name'] = self.want.profile_name
        params['partition'] = self.want.partition
        uri = "https://{0}:{1}/mgmt/tm/security/log/profile/{2}/network/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.profile_name)
        )
        resp = self.client.api.post(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] in [400, 409]:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return True

    def update_on_device(self):
        params = self.changes.api_params()
        uri = "https://{0}:{1}/mgmt/tm/security/log/profile/{2}/network/{3}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.profile_name),
            self._internal_name()
        )
        resp = self.client.api.patch(uri, json=params)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)

    def remove_from_device(self):
        uri = "https://{0}:{1}/mgmt/tm/security/log/profile/{2}/network/{3}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.profile_name),
            self._internal_name()
        )
        response = self.client.api.delete(uri)
        if response.status == 200:
            return True
        raise F5ModuleError(response.content)

    def read_current_from_device(self):
        uri = "https://{0}:{1}/mgmt/tm/security/log/profile/{2}/network/{3}".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
            transform_name(self.want.partition, self.want.profile_name),
            self._internal_name()
        )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return ApiParameters(params=response)


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = True
        self.choices = [
            'acl_policy_name', 'acl_policy_type', 'acl_rule_name', 'action',
            'bigip_hostname', 'context_name', 'context_type', 'date_time',
            'dest_fqdn', 'dest_geo', 'dest_ip', 'dest_port', 'drop_reason',
            'management_ip_address', 'protocol', 'route_domain', 'sa_translation_pool',
            'sa_translation_type', 'source_fqdn', 'source_user', 'src_geo', 'src_ip',
            'src_port', 'translated_dest_ip', 'translated_dest_port', 'translated_ip_protocol',
            'translated_route_domain', 'translated_src_ip', 'translated_src_port', 'translated_vlan',
            'vlan'
        ]
        argument_spec = dict(
            profile_name=dict(
                required=True
            ),
            rate_limit=dict(),
            log_publisher=dict(),
            log_matches_accept_rule=dict(
                type='dict',
                options=dict(
                    enabled=dict(type='bool'),
                    rate_limit=dict()
                )
            ),
            log_matches_drop_rule=dict(
                type='dict',
                options=dict(
                    enabled=dict(type='bool'),
                    rate_limit=dict()
                )
            ),
            log_matches_reject_rule=dict(
                type='dict',
                options=dict(
                    enabled=dict(type='bool'),
                    rate_limit=dict()
                )
            ),
            log_tcp_errors=dict(
                type='dict',
                options=dict(
                    enabled=dict(type='bool'),
                    rate_limit=dict()
                )
            ),
            log_tcp_events=dict(
                type='dict',
                options=dict(
                    enabled=dict(type='bool'),
                    rate_limit=dict()
                )
            ),
            log_ip_errors=dict(
                type='dict',
                options=dict(
                    enabled=dict(type='bool'),
                    rate_limit=dict()
                )
            ),
            log_translation_fields=dict(type='bool'),
            log_storage_format=dict(
                choices=['none', 'field-list']
            ),
            log_format_delimiter=dict(),
            log_message_fields=dict(
                type='list',
                choices=self.choices
            ),
            partition=dict(
                default='Common',
                fallback=(env_fallback, ['F5_PARTITION'])
            ),
            state=dict(
                default='present',
                choices=['present', 'absent']
            )
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode,
    )

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''

module: docker_compose

short_description: Manage multi-container Docker applications with Docker Compose.

version_added: "2.1"

author: "Chris Houseknecht (@chouseknecht)"

description:
  - Uses Docker Compose to start, shutdown and scale services.
  - Works with compose versions 1 and 2.
  - Configuration can be read from a C(docker-compose.yml) or C(docker-compose.yaml) file or inline using the I(definition) option.
  - See the examples for more details.
  - Supports check mode.
  - This module was called C(docker_service) before Ansible 2.8. The usage did not change.

options:
  project_src:
    description:
      - Path to a directory containing a C(docker-compose.yml) or C(docker-compose.yaml) file.
      - Mutually exclusive with I(definition).
      - Required when no I(definition) is provided.
    type: path
  project_name:
    description:
      - Provide a project name. If not provided, the project name is taken from the basename of I(project_src).
      - Required when I(definition) is provided.
    type: str
  files:
    description:
      - List of Compose file names relative to I(project_src). Overrides C(docker-compose.yml) or C(docker-compose.yaml).
      - Files are loaded and merged in the order given.
    type: list
    elements: path
  state:
    description:
      - Desired state of the project.
      - Specifying C(present) is the same as running C(docker-compose up) resp. C(docker-compose stop) (with I(stopped)) resp. C(docker-compose restart)
        (with I(restarted)).
      - Specifying C(absent) is the same as running C(docker-compose down).
    type: str
    default: present
    choices:
      - absent
      - present
  services:
    description:
      - When I(state) is C(present) run C(docker-compose up) resp. C(docker-compose stop) (with I(stopped)) resp. C(docker-compose restart) (with I(restarted))
        on a subset of services.
      - If empty, which is the default, the operation will be performed on all services defined in the Compose file (or inline I(definition)).
    type: list
    elements: str
  scale:
    description:
      - When I(state) is C(present) scale services. Provide a dictionary of key/value pairs where the key
        is the name of the service and the value is an integer count for the number of containers.
    type: dict
  dependencies:
    description:
      - When I(state) is C(present) specify whether or not to include linked services.
    type: bool
    default: yes
  definition:
    description:
      - Compose file describing one or more services, networks and volumes.
      - Mutually exclusive with I(project_src) and I(files).
    type: dict
  hostname_check:
    description:
      - Whether or not to check the Docker daemon's hostname against the name provided in the client certificate.
    type: bool
    default: no
  recreate:
    description:
      - By default containers will be recreated when their configuration differs from the service definition.
      - Setting to C(never) ignores configuration differences and leaves existing containers unchanged.
      - Setting to C(always) forces recreation of all existing containers.
    type: str
    default: smart
    choices:
      - always
      - never
      - smart
  build:
    description:
      - Use with I(state) C(present) to always build images prior to starting the application.
      - Same as running C(docker-compose build) with the pull option.
      - Images will only be rebuilt if Docker detects a change in the Dockerfile or build directory contents.
      - Use the I(nocache) option to ignore the image cache when performing the build.
      - If an existing image is replaced, services using the image will be recreated unless I(recreate) is C(never).
    type: bool
    default: no
  pull:
    description:
      - Use with I(state) C(present) to always pull images prior to starting the application.
      - Same as running C(docker-compose pull).
      - When a new image is pulled, services using the image will be recreated unless I(recreate) is C(never).
    type: bool
    default: no
    version_added: "2.2"
  nocache:
    description:
      - Use with the I(build) option to ignore the cache during the image build process.
    type: bool
    default: no
    version_added: "2.2"
  remove_images:
    description:
      - Use with I(state) C(absent) to remove all images or only local images.
    type: str
    choices:
        - 'all'
        - 'local'
  remove_volumes:
    description:
      - Use with I(state) C(absent) to remove data volumes.
    type: bool
    default: no
  stopped:
    description:
      - Use with I(state) C(present) to stop all containers defined in the Compose file.
      - If I(services) is defined, only the containers listed there will be stopped.
    type: bool
    default: no
  restarted:
    description:
      - Use with I(state) C(present) to restart all containers defined in the Compose file.
      - If I(services) is defined, only the containers listed there will be restarted.
    type: bool
    default: no
  remove_orphans:
    description:
      - Remove containers for services not defined in the Compose file.
    type: bool
    default: no
  timeout:
    description:
        - timeout in seconds for container shutdown when attached or when containers are already running.
    type: int
    default: 10

extends_documentation_fragment:
  - docker
  - docker.docker_py_1_documentation

requirements:
  - "L(Docker SDK for Python,https://docker-py.readthedocs.io/en/stable/) >= 1.8.0 (use L(docker-py,https://pypi.org/project/docker-py/) for Python 2.6)"
  - "docker-compose >= 1.7.0"
  - "Docker API >= 1.20"
  - "PyYAML >= 3.11"
'''

EXAMPLES = '''
# Examples use the django example at https://docs.docker.com/compose/django. Follow it to create the
# flask directory

- name: Run using a project directory
  hosts: localhost
  gather_facts: no
  tasks:
    - name: Tear down existing services
      docker_compose:
        project_src: flask
        state: absent

    - name: Create and start services
      docker_compose:
        project_src: flask
      register: output

    - debug:
        var: output

    - name: Run `docker-compose up` again
      docker_compose:
        project_src: flask
        build: no
      register: output

    - debug:
        var: output

    - assert:
        that: "not output.changed "

    - name: Stop all services
      docker_compose:
        project_src: flask
        build: no
        stopped: yes
      register: output

    - debug:
        var: output

    - assert:
        that:
          - "not web.flask_web_1.state.running"
          - "not db.flask_db_1.state.running"

    - name: Restart services
      docker_compose:
        project_src: flask
        build: no
        restarted: yes
      register: output

    - debug:
        var: output

    - assert:
        that:
          - "web.flask_web_1.state.running"
          - "db.flask_db_1.state.running"

- name: Scale the web service to 2
  hosts: localhost
  gather_facts: no
  tasks:
    - docker_compose:
        project_src: flask
        scale:
          web: 2
      register: output

    - debug:
        var: output

- name: Run with inline v2 compose
  hosts: localhost
  gather_facts: no
  tasks:
    - docker_compose:
        project_src: flask
        state: absent

    - docker_compose:
        project_name: flask
        definition:
          version: '2'
          services:
            db:
              image: postgres
            web:
              build: "{{ playbook_dir }}/flask"
              command: "python manage.py runserver 0.0.0.0:8000"
              volumes:
                - "{{ playbook_dir }}/flask:/code"
              ports:
                - "8000:8000"
              depends_on:
                - db
      register: output

    - debug:
        var: output

    - assert:
        that:
          - "web.flask_web_1.state.running"
          - "db.flask_db_1.state.running"

- name: Run with inline v1 compose
  hosts: localhost
  gather_facts: no
  tasks:
    - docker_compose:
        project_src: flask
        state: absent

    - docker_compose:
        project_name: flask
        definition:
            db:
              image: postgres
            web:
              build: "{{ playbook_dir }}/flask"
              command: "python manage.py runserver 0.0.0.0:8000"
              volumes:
                - "{{ playbook_dir }}/flask:/code"
              ports:
                - "8000:8000"
              links:
                - db
      register: output

    - debug:
        var: output

    - assert:
        that:
          - "web.flask_web_1.state.running"
          - "db.flask_db_1.state.running"
'''

RETURN = '''
services:
  description:
  - A dictionary mapping the service's name to a dictionary of containers.
  - Note that facts are part of the registered vars since Ansible 2.8. For compatibility reasons, the facts
    are also accessible directly. The service's name is the variable with which the container dictionary
    can be accessed. Note that the returned facts will be removed in Ansible 2.12.
  returned: success
  type: complex
  contains:
      container_name:
          description: Name of the container. Format is C(project_service_#).
          returned: success
          type: complex
          contains:
              cmd:
                  description: One or more commands to be executed in the container.
                  returned: success
                  type: list
                  elements: str
                  example: ["postgres"]
              image:
                  description: Name of the image from which the container was built.
                  returned: success
                  type: str
                  example: postgres
              labels:
                  description: Meta data assigned to the container.
                  returned: success
                  type: dict
                  example: {...}
              networks:
                  description: Contains a dictionary for each network to which the container is a member.
                  returned: success
                  type: list
                  elements: dict
                  contains:
                      IPAddress:
                          description: The IP address assigned to the container.
                          returned: success
                          type: str
                          example: 172.17.0.2
                      IPPrefixLen:
                          description: Number of bits used by the subnet.
                          returned: success
                          type: int
                          example: 16
                      aliases:
                          description: Aliases assigned to the container by the network.
                          returned: success
                          type: list
                          elements: str
                          example: ['db']
                      globalIPv6:
                          description: IPv6 address assigned to the container.
                          returned: success
                          type: str
                          example: ''
                      globalIPv6PrefixLen:
                          description: IPv6 subnet length.
                          returned: success
                          type: int
                          example: 0
                      links:
                          description: List of container names to which this container is linked.
                          returned: success
                          type: list
                          elements: str
                          example: null
                      macAddress:
                          description: Mac Address assigned to the virtual NIC.
                          returned: success
                          type: str
                          example: "02:42:ac:11:00:02"
              state:
                  description: Information regarding the current disposition of the container.
                  returned: success
                  type: dict
                  contains:
                      running:
                          description: Whether or not the container is up with a running process.
                          returned: success
                          type: bool
                          example: true
                      status:
                          description: Description of the running state.
                          returned: success
                          type: str
                          example: running

actions:
  description: Provides the actions to be taken on each service as determined by compose.
  returned: when in check mode or I(debug) is C(yes)
  type: complex
  contains:
      service_name:
          description: Name of the service.
          returned: always
          type: complex
          contains:
              pulled_image:
                  description: Provides image details when a new image is pulled for the service.
                  returned: on image pull
                  type: complex
                  contains:
                      name:
                          description: name of the image
                          returned: always
                          type: str
                      id:
                          description: image hash
                          returned: always
                          type: str
              built_image:
                  description: Provides image details when a new image is built for the service.
                  returned: on image build
                  type: complex
                  contains:
                      name:
                          description: name of the image
                          returned: always
                          type: str
                      id:
                          description: image hash
                          returned: always
                          type: str

              action:
                  description: A descriptive name of the action to be performed on the service's containers.
                  returned: always
                  type: list
                  elements: str
                  contains:
                      id:
                          description: the container's long ID
                          returned: always
                          type: str
                      name:
                          description: the container's name
                          returned: always
                          type: str
                      short_id:
                          description: the container's short ID
                          returned: always
                          type: str
'''

import os
import re
import sys
import tempfile
import traceback
from contextlib import contextmanager
from distutils.version import LooseVersion

try:
    import yaml
    HAS_YAML = True
    HAS_YAML_EXC = None
except ImportError as dummy:
    HAS_YAML = False
    HAS_YAML_EXC = traceback.format_exc()

try:
    from docker.errors import DockerException
except ImportError:
    # missing Docker SDK for Python handled in ansible.module_utils.docker.common
    pass

try:
    from compose import __version__ as compose_version
    from compose.cli.command import project_from_options
    from compose.service import NoSuchImageError
    from compose.cli.main import convergence_strategy_from_opts, build_action_from_opts, image_type_from_opt
    from compose.const import DEFAULT_TIMEOUT, LABEL_SERVICE, LABEL_PROJECT, LABEL_ONE_OFF
    HAS_COMPOSE = True
    HAS_COMPOSE_EXC = None
    MINIMUM_COMPOSE_VERSION = '1.7.0'
except ImportError as dummy:
    HAS_COMPOSE = False
    HAS_COMPOSE_EXC = traceback.format_exc()
    DEFAULT_TIMEOUT = 10

from ansible.module_utils.docker.common import (
    AnsibleDockerClient,
    DockerBaseClass,
    RequestException,
)


AUTH_PARAM_MAPPING = {
    u'docker_host': u'--host',
    u'tls': u'--tls',
    u'cacert_path': u'--tlscacert',
    u'cert_path': u'--tlscert',
    u'key_path': u'--tlskey',
    u'tls_verify': u'--tlsverify'
}


@contextmanager
def stdout_redirector(path_name):
    old_stdout = sys.stdout
    fd = open(path_name, 'w')
    sys.stdout = fd
    try:
        yield
    finally:
        sys.stdout = old_stdout


@contextmanager
def stderr_redirector(path_name):
    old_fh = sys.stderr
    fd = open(path_name, 'w')
    sys.stderr = fd
    try:
        yield
    finally:
        sys.stderr = old_fh


def make_redirection_tempfiles():
    dummy, out_redir_name = tempfile.mkstemp(prefix="ansible")
    dummy, err_redir_name = tempfile.mkstemp(prefix="ansible")
    return (out_redir_name, err_redir_name)


def cleanup_redirection_tempfiles(out_name, err_name):
    for i in [out_name, err_name]:
        os.remove(i)


def get_redirected_output(path_name):
    output = []
    with open(path_name, 'r') as fd:
        for line in fd:
            # strip terminal format/color chars
            new_line = re.sub(r'\x1b\[.+m', '', line)
            output.append(new_line)
    os.remove(path_name)
    return output


def attempt_extract_errors(exc_str, stdout, stderr):
    errors = [l.strip() for l in stderr if l.strip().startswith('ERROR:')]
    errors.extend([l.strip() for l in stdout if l.strip().startswith('ERROR:')])

    warnings = [l.strip() for l in stderr if l.strip().startswith('WARNING:')]
    warnings.extend([l.strip() for l in stdout if l.strip().startswith('WARNING:')])

    # assume either the exception body (if present) or the last warning was the 'most'
    # fatal.

    if exc_str.strip():
        msg = exc_str.strip()
    elif errors:
        msg = errors[-1].encode('utf-8')
    else:
        msg = 'unknown cause'

    return {
        'warnings': [w.encode('utf-8') for w in warnings],
        'errors': [e.encode('utf-8') for e in errors],
        'msg': msg,
        'module_stderr': ''.join(stderr),
        'module_stdout': ''.join(stdout)
    }


def get_failure_info(exc, out_name, err_name=None, msg_format='%s'):
    if err_name is None:
        stderr = []
    else:
        stderr = get_redirected_output(err_name)
    stdout = get_redirected_output(out_name)

    reason = attempt_extract_errors(str(exc), stdout, stderr)
    reason['msg'] = msg_format % reason['msg']
    return reason


class ContainerManager(DockerBaseClass):

    def __init__(self, client):

        super(ContainerManager, self).__init__()

        self.client = client
        self.project_src = None
        self.files = None
        self.project_name = None
        self.state = None
        self.definition = None
        self.hostname_check = None
        self.timeout = None
        self.remove_images = None
        self.remove_orphans = None
        self.remove_volumes = None
        self.stopped = None
        self.restarted = None
        self.recreate = None
        self.build = None
        self.dependencies = None
        self.services = None
        self.scale = None
        self.debug = None
        self.pull = None
        self.nocache = None

        for key, value in client.module.params.items():
            setattr(self, key, value)

        self.check_mode = client.check_mode

        if not self.debug:
            self.debug = client.module._debug

        self.options = dict()
        self.options.update(self._get_auth_options())
        self.options[u'--skip-hostname-check'] = (not self.hostname_check)

        if self.project_name:
            self.options[u'--project-name'] = self.project_name

        if self.files:
            self.options[u'--file'] = self.files

        if not HAS_COMPOSE:
            self.client.fail("Unable to load docker-compose. Try `pip install docker-compose`. Error: %s" %
                             HAS_COMPOSE_EXC)

        if LooseVersion(compose_version) < LooseVersion(MINIMUM_COMPOSE_VERSION):
            self.client.fail("Found docker-compose version %s. Minimum required version is %s. "
                             "Upgrade docker-compose to a min version of %s." %
                             (compose_version, MINIMUM_COMPOSE_VERSION, MINIMUM_COMPOSE_VERSION))

        self.log("options: ")
        self.log(self.options, pretty_print=True)

        if self.definition:
            if not HAS_YAML:
                self.client.fail("Unable to load yaml. Try `pip install PyYAML`. Error: %s" % HAS_YAML_EXC)

            if not self.project_name:
                self.client.fail("Parameter error - project_name required when providing definition.")

            self.project_src = tempfile.mkdtemp(prefix="ansible")
            compose_file = os.path.join(self.project_src, "docker-compose.yml")
            try:
                self.log('writing: ')
                self.log(yaml.dump(self.definition, default_flow_style=False))
                with open(compose_file, 'w') as f:
                    f.write(yaml.dump(self.definition, default_flow_style=False))
            except Exception as exc:
                self.client.fail("Error writing to %s - %s" % (compose_file, str(exc)))
        else:
            if not self.project_src:
                self.client.fail("Parameter error - project_src required.")

        try:
            self.log("project_src: %s" % self.project_src)
            self.project = project_from_options(self.project_src, self.options)
        except Exception as exc:
            self.client.fail("Configuration error - %s" % str(exc))

    def exec_module(self):
        result = dict()

        if self.state == 'present':
            result = self.cmd_up()
        elif self.state == 'absent':
            result = self.cmd_down()

        if self.definition:
            compose_file = os.path.join(self.project_src, "docker-compose.yml")
            self.log("removing %s" % compose_file)
            os.remove(compose_file)
            self.log("removing %s" % self.project_src)
            os.rmdir(self.project_src)

        if not self.check_mode and not self.debug and result.get('actions'):
            result.pop('actions')

        return result

    def _get_auth_options(self):
        options = dict()
        for key, value in self.client.auth_params.items():
            if value is not None:
                option = AUTH_PARAM_MAPPING.get(key)
                if option:
                    options[option] = value
        return options

    def cmd_up(self):

        start_deps = self.dependencies
        service_names = self.services
        detached = True
        result = dict(changed=False, actions=[], ansible_facts=dict(), services=dict())

        up_options = {
            u'--no-recreate': False,
            u'--build': False,
            u'--no-build': False,
            u'--no-deps': False,
            u'--force-recreate': False,
        }

        if self.recreate == 'never':
            up_options[u'--no-recreate'] = True
        elif self.recreate == 'always':
            up_options[u'--force-recreate'] = True

        if self.remove_orphans:
            up_options[u'--remove-orphans'] = True

        converge = convergence_strategy_from_opts(up_options)
        self.log("convergence strategy: %s" % converge)

        if self.pull:
            pull_output = self.cmd_pull()
            result['changed'] = pull_output['changed']
            result['actions'] += pull_output['actions']

        if self.build:
            build_output = self.cmd_build()
            result['changed'] = build_output['changed']
            result['actions'] += build_output['actions']

        if self.remove_orphans:
            containers = self.client.containers(
                filters={
                    'label': [
                        '{0}={1}'.format(LABEL_PROJECT, self.project.name),
                        '{0}={1}'.format(LABEL_ONE_OFF, "False")
                    ],
                }
            )

            orphans = []
            for container in containers:
                service_name = container.get('Labels', {}).get(LABEL_SERVICE)
                if service_name not in self.project.service_names:
                    orphans.append(service_name)

            if orphans:
                result['changed'] = True

        for service in self.project.services:
            if not service_names or service.name in service_names:
                plan = service.convergence_plan(strategy=converge)
                if plan.action != 'noop':
                    result['changed'] = True
                    result_action = dict(service=service.name)
                    result_action[plan.action] = []
                    for container in plan.containers:
                        result_action[plan.action].append(dict(
                            id=container.id,
                            name=container.name,
                            short_id=container.short_id,
                        ))
                    result['actions'].append(result_action)

        if not self.check_mode and result['changed']:
            out_redir_name, err_redir_name = make_redirection_tempfiles()
            try:
                with stdout_redirector(out_redir_name):
                    with stderr_redirector(err_redir_name):
                        do_build = build_action_from_opts(up_options)
                        self.log('Setting do_build to %s' % do_build)
                        self.project.up(
                            service_names=service_names,
                            start_deps=start_deps,
                            strategy=converge,
                            do_build=do_build,
                            detached=detached,
                            remove_orphans=self.remove_orphans,
                            timeout=self.timeout)
            except Exception as exc:
                fail_reason = get_failure_info(exc, out_redir_name, err_redir_name,
                                               msg_format="Error starting project %s")
                self.client.fail(**fail_reason)
            else:
                cleanup_redirection_tempfiles(out_redir_name, err_redir_name)

        if self.stopped:
            stop_output = self.cmd_stop(service_names)
            result['changed'] = stop_output['changed']
            result['actions'] += stop_output['actions']

        if self.restarted:
            restart_output = self.cmd_restart(service_names)
            result['changed'] = restart_output['changed']
            result['actions'] += restart_output['actions']

        if self.scale:
            scale_output = self.cmd_scale()
            result['changed'] = scale_output['changed']
            result['actions'] += scale_output['actions']

        for service in self.project.services:
            service_facts = dict()
            result['ansible_facts'][service.name] = service_facts
            result['services'][service.name] = service_facts
            for container in service.containers(stopped=True):
                inspection = container.inspect()
                # pare down the inspection data to the most useful bits
                facts = dict(
                    cmd=[],
                    labels=dict(),
                    image=None,
                    state=dict(
                        running=None,
                        status=None
                    ),
                    networks=dict()
                )
                if inspection['Config'].get('Cmd', None) is not None:
                    facts['cmd'] = inspection['Config']['Cmd']
                if inspection['Config'].get('Labels', None) is not None:
                    facts['labels'] = inspection['Config']['Labels']
                if inspection['Config'].get('Image', None) is not None:
                    facts['image'] = inspection['Config']['Image']
                if inspection['State'].get('Running', None) is not None:
                    facts['state']['running'] = inspection['State']['Running']
                if inspection['State'].get('Status', None) is not None:
                    facts['state']['status'] = inspection['State']['Status']

                if inspection.get('NetworkSettings') and inspection['NetworkSettings'].get('Networks'):
                    networks = inspection['NetworkSettings']['Networks']
                    for key in networks:
                        facts['networks'][key] = dict(
                            aliases=[],
                            globalIPv6=None,
                            globalIPv6PrefixLen=0,
                            IPAddress=None,
                            IPPrefixLen=0,
                            links=None,
                            macAddress=None,
                        )
                        if networks[key].get('Aliases', None) is not None:
                            facts['networks'][key]['aliases'] = networks[key]['Aliases']
                        if networks[key].get('GlobalIPv6Address', None) is not None:
                            facts['networks'][key]['globalIPv6'] = networks[key]['GlobalIPv6Address']
                        if networks[key].get('GlobalIPv6PrefixLen', None) is not None:
                            facts['networks'][key]['globalIPv6PrefixLen'] = networks[key]['GlobalIPv6PrefixLen']
                        if networks[key].get('IPAddress', None) is not None:
                            facts['networks'][key]['IPAddress'] = networks[key]['IPAddress']
                        if networks[key].get('IPPrefixLen', None) is not None:
                            facts['networks'][key]['IPPrefixLen'] = networks[key]['IPPrefixLen']
                        if networks[key].get('Links', None) is not None:
                            facts['networks'][key]['links'] = networks[key]['Links']
                        if networks[key].get('MacAddress', None) is not None:
                            facts['networks'][key]['macAddress'] = networks[key]['MacAddress']

                service_facts[container.name] = facts

        return result

    def cmd_pull(self):
        result = dict(
            changed=False,
            actions=[],
        )

        if not self.check_mode:
            for service in self.project.get_services(self.services, include_deps=False):
                if 'image' not in service.options:
                    continue

                self.log('Pulling image for service %s' % service.name)
                # store the existing image ID
                old_image_id = ''
                try:
                    image = service.image()
                    if image and image.get('Id'):
                        old_image_id = image['Id']
                except NoSuchImageError:
                    pass
                except Exception as exc:
                    self.client.fail("Error: service image lookup failed - %s" % str(exc))

                out_redir_name, err_redir_name = make_redirection_tempfiles()
                # pull the image
                try:
                    with stdout_redirector(out_redir_name):
                        with stderr_redirector(err_redir_name):
                            service.pull(ignore_pull_failures=False)
                except Exception as exc:
                    fail_reason = get_failure_info(exc, out_redir_name, err_redir_name,
                                                   msg_format="Error: pull failed with %s")
                    self.client.fail(**fail_reason)
                else:
                    cleanup_redirection_tempfiles(out_redir_name, err_redir_name)

                # store the new image ID
                new_image_id = ''
                try:
                    image = service.image()
                    if image and image.get('Id'):
                        new_image_id = image['Id']
                except NoSuchImageError as exc:
                    self.client.fail("Error: service image lookup failed after pull - %s" % str(exc))

                if new_image_id != old_image_id:
                    # if a new image was pulled
                    result['changed'] = True
                    result['actions'].append(dict(
                        service=service.name,
                        pulled_image=dict(
                            name=service.image_name,
                            id=new_image_id
                        )
                    ))
        return result

    def cmd_build(self):
        result = dict(
            changed=False,
            actions=[]
        )
        if not self.check_mode:
            for service in self.project.get_services(self.services, include_deps=False):
                if service.can_be_built():
                    self.log('Building image for service %s' % service.name)
                    # store the existing image ID
                    old_image_id = ''
                    try:
                        image = service.image()
                        if image and image.get('Id'):
                            old_image_id = image['Id']
                    except NoSuchImageError:
                        pass
                    except Exception as exc:
                        self.client.fail("Error: service image lookup failed - %s" % str(exc))

                    out_redir_name, err_redir_name = make_redirection_tempfiles()
                    # build the image
                    try:
                        with stdout_redirector(out_redir_name):
                            with stderr_redirector(err_redir_name):
                                new_image_id = service.build(pull=self.pull, no_cache=self.nocache)
                    except Exception as exc:
                        fail_reason = get_failure_info(exc, out_redir_name, err_redir_name,
                                                       msg_format="Error: build failed with %s")
                        self.client.fail(**fail_reason)
                    else:
                        cleanup_redirection_tempfiles(out_redir_name, err_redir_name)

                    if new_image_id not in old_image_id:
                        # if a new image was built
                        result['changed'] = True
                        result['actions'].append(dict(
                            service=service.name,
                            built_image=dict(
                                name=service.image_name,
                                id=new_image_id
                            )
                        ))
        return result

    def cmd_down(self):
        result = dict(
            changed=False,
            actions=[]
        )
        for service in self.project.services:
            containers = service.containers(stopped=True)
            if len(containers):
                result['changed'] = True
            result['actions'].append(dict(
                service=service.name,
                deleted=[container.name for container in containers]
            ))
        if not self.check_mode and result['changed']:
            image_type = image_type_from_opt('--rmi', self.remove_images)
            out_redir_name, err_redir_name = make_redirection_tempfiles()
            try:
                with stdout_redirector(out_redir_name):
                    with stderr_redirector(err_redir_name):
                        self.project.down(image_type, self.remove_volumes, self.remove_orphans)
            except Exception as exc:
                fail_reason = get_failure_info(exc, out_redir_name, err_redir_name,
                                               msg_format="Error stopping project - %s")
                self.client.fail(**fail_reason)
            else:
                cleanup_redirection_tempfiles(out_redir_name, err_redir_name)
        return result

    def cmd_stop(self, service_names):
        result = dict(
            changed=False,
            actions=[]
        )
        for service in self.project.services:
            if not service_names or service.name in service_names:
                service_res = dict(
                    service=service.name,
                    stop=[]
                )
                for container in service.containers(stopped=False):
                    result['changed'] = True
                    service_res['stop'].append(dict(
                        id=container.id,
                        name=container.name,
                        short_id=container.short_id
                    ))
                result['actions'].append(service_res)
        if not self.check_mode and result['changed']:
            out_redir_name, err_redir_name = make_redirection_tempfiles()
            try:
                with stdout_redirector(out_redir_name):
                    with stderr_redirector(err_redir_name):
                        self.project.stop(service_names=service_names, timeout=self.timeout)
            except Exception as exc:
                fail_reason = get_failure_info(exc, out_redir_name, err_redir_name,
                                               msg_format="Error stopping project %s")
                self.client.fail(**fail_reason)
            else:
                cleanup_redirection_tempfiles(out_redir_name, err_redir_name)
        return result

    def cmd_restart(self, service_names):
        result = dict(
            changed=False,
            actions=[]
        )

        for service in self.project.services:
            if not service_names or service.name in service_names:
                service_res = dict(
                    service=service.name,
                    restart=[]
                )
                for container in service.containers(stopped=True):
                    result['changed'] = True
                    service_res['restart'].append(dict(
                        id=container.id,
                        name=container.name,
                        short_id=container.short_id
                    ))
                result['actions'].append(service_res)

        if not self.check_mode and result['changed']:
            out_redir_name, err_redir_name = make_redirection_tempfiles()
            try:
                with stdout_redirector(out_redir_name):
                    with stderr_redirector(err_redir_name):
                        self.project.restart(service_names=service_names, timeout=self.timeout)
            except Exception as exc:
                fail_reason = get_failure_info(exc, out_redir_name, err_redir_name,
                                               msg_format="Error restarting project %s")
                self.client.fail(**fail_reason)
            else:
                cleanup_redirection_tempfiles(out_redir_name, err_redir_name)
        return result

    def cmd_scale(self):
        result = dict(
            changed=False,
            actions=[]
        )
        for service in self.project.services:
            if service.name in self.scale:
                service_res = dict(
                    service=service.name,
                    scale=0
                )
                containers = service.containers(stopped=True)
                scale = self.parse_scale(service.name)
                if len(containers) != scale:
                    result['changed'] = True
                    service_res['scale'] = scale - len(containers)
                    if not self.check_mode:
                        out_redir_name, err_redir_name = make_redirection_tempfiles()
                        try:
                            with stdout_redirector(out_redir_name):
                                with stderr_redirector(err_redir_name):
                                    service.scale(scale)
                        except Exception as exc:
                            fail_reason = get_failure_info(exc, out_redir_name, err_redir_name,
                                                           msg_format="Error scaling {0} - %s".format(service.name))
                            self.client.fail(**fail_reason)
                        else:
                            cleanup_redirection_tempfiles(out_redir_name, err_redir_name)
                    result['actions'].append(service_res)
        return result

    def parse_scale(self, service_name):
        try:
            return int(self.scale[service_name])
        except ValueError:
            self.client.fail("Error scaling %s - expected int, got %s",
                             service_name, str(type(self.scale[service_name])))


def main():
    argument_spec = dict(
        project_src=dict(type='path'),
        project_name=dict(type='str',),
        files=dict(type='list', elements='path'),
        state=dict(type='str', default='present', choices=['absent', 'present']),
        definition=dict(type='dict'),
        hostname_check=dict(type='bool', default=False),
        recreate=dict(type='str', default='smart', choices=['always', 'never', 'smart']),
        build=dict(type='bool', default=False),
        remove_images=dict(type='str', choices=['all', 'local']),
        remove_volumes=dict(type='bool', default=False),
        remove_orphans=dict(type='bool', default=False),
        stopped=dict(type='bool', default=False),
        restarted=dict(type='bool', default=False),
        scale=dict(type='dict'),
        services=dict(type='list', elements='str'),
        dependencies=dict(type='bool', default=True),
        pull=dict(type='bool', default=False),
        nocache=dict(type='bool', default=False),
        debug=dict(type='bool', default=False),
        timeout=dict(type='int', default=DEFAULT_TIMEOUT)
    )

    mutually_exclusive = [
        ('definition', 'project_src'),
        ('definition', 'files')
    ]

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
        min_docker_api_version='1.20',
    )
    if client.module._name == 'docker_service':
        client.module.deprecate("The 'docker_service' module has been renamed to 'docker_compose'.", version='2.12')

    try:
        result = ContainerManager(client).exec_module()
        client.module.exit_json(**result)
    except DockerException as e:
        client.fail('An unexpected docker error occurred: {0}'.format(e), exception=traceback.format_exc())
    except RequestException as e:
        client.fail('An unexpected requests error occurred when docker-py tried to talk to the docker daemon: {0}'.format(e), exception=traceback.format_exc())


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Google
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file at
#     https://www.github.com/GoogleCloudPlatform/magic-modules
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function

__metaclass__ = type

################################################################################
# Documentation
################################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ["preview"], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: gcp_compute_target_https_proxy
description:
- Represents a TargetHttpsProxy resource, which is used by one or more global forwarding
  rule to route incoming HTTPS requests to a URL map.
short_description: Creates a GCP TargetHttpsProxy
version_added: 2.6
author: Google Inc. (@googlecloudplatform)
requirements:
- python >= 2.6
- requests >= 2.18.4
- google-auth >= 1.3.0
options:
  state:
    description:
    - Whether the given object should exist in GCP
    choices:
    - present
    - absent
    default: present
    type: str
  description:
    description:
    - An optional description of this resource.
    required: false
    type: str
  name:
    description:
    - Name of the resource. Provided by the client when the resource is created. The
      name must be 1-63 characters long, and comply with RFC1035. Specifically, the
      name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`
      which means the first character must be a lowercase letter, and all following
      characters must be a dash, lowercase letter, or digit, except the last character,
      which cannot be a dash.
    required: true
    type: str
  quic_override:
    description:
    - Specifies the QUIC override policy for this resource. This determines whether
      the load balancer will attempt to negotiate QUIC with clients or not. Can specify
      one of NONE, ENABLE, or DISABLE. If NONE is specified, uses the QUIC policy
      with no user overrides, which is equivalent to DISABLE. Not specifying this
      field is equivalent to specifying NONE.
    - 'Some valid choices include: "NONE", "ENABLE", "DISABLE"'
    required: false
    type: str
    version_added: 2.7
  ssl_certificates:
    description:
    - A list of SslCertificate resources that are used to authenticate connections
      between users and the load balancer. Currently, exactly one SSL certificate
      must be specified.
    required: true
    type: list
  ssl_policy:
    description:
    - A reference to the SslPolicy resource that will be associated with the TargetHttpsProxy
      resource. If not set, the TargetHttpsProxy resource will not have any SSL policy
      configured.
    - 'This field represents a link to a SslPolicy resource in GCP. It can be specified
      in two ways. First, you can place a dictionary with key ''selfLink'' and value
      of your resource''s selfLink Alternatively, you can add `register: name-of-resource`
      to a gcp_compute_ssl_policy task and then set this ssl_policy field to "{{ name-of-resource
      }}"'
    required: false
    type: dict
    version_added: 2.8
  url_map:
    description:
    - A reference to the UrlMap resource that defines the mapping from URL to the
      BackendService.
    - 'This field represents a link to a UrlMap resource in GCP. It can be specified
      in two ways. First, you can place a dictionary with key ''selfLink'' and value
      of your resource''s selfLink Alternatively, you can add `register: name-of-resource`
      to a gcp_compute_url_map task and then set this url_map field to "{{ name-of-resource
      }}"'
    required: true
    type: dict
extends_documentation_fragment: gcp
notes:
- 'API Reference: U(https://cloud.google.com/compute/docs/reference/v1/targetHttpsProxies)'
- 'Official Documentation: U(https://cloud.google.com/compute/docs/load-balancing/http/target-proxies)'
'''

EXAMPLES = '''
- name: create a instance group
  gcp_compute_instance_group:
    name: instancegroup-targethttpsproxy
    zone: us-central1-a
    project: "{{ gcp_project }}"
    auth_kind: "{{ gcp_cred_kind }}"
    service_account_file: "{{ gcp_cred_file }}"
    state: present
  register: instancegroup

- name: create a HTTP health check
  gcp_compute_http_health_check:
    name: httphealthcheck-targethttpsproxy
    healthy_threshold: 10
    port: 8080
    timeout_sec: 2
    unhealthy_threshold: 5
    project: "{{ gcp_project }}"
    auth_kind: "{{ gcp_cred_kind }}"
    service_account_file: "{{ gcp_cred_file }}"
    state: present
  register: healthcheck

- name: create a backend service
  gcp_compute_backend_service:
    name: backendservice-targethttpsproxy
    backends:
    - group: "{{ instancegroup.selfLink }}"
    health_checks:
    - "{{ healthcheck.selfLink }}"
    enable_cdn: 'true'
    project: "{{ gcp_project }}"
    auth_kind: "{{ gcp_cred_kind }}"
    service_account_file: "{{ gcp_cred_file }}"
    state: present
  register: backendservice

- name: create a URL map
  gcp_compute_url_map:
    name: urlmap-targethttpsproxy
    default_service: "{{ backendservice }}"
    project: "{{ gcp_project }}"
    auth_kind: "{{ gcp_cred_kind }}"
    service_account_file: "{{ gcp_cred_file }}"
    state: present
  register: urlmap

- name: create a SSL certificate
  gcp_compute_ssl_certificate:
    name: sslcert-targethttpsproxy
    description: A certificate for testing. Do not use this certificate in production
    certificate: |-
      -----BEGIN CERTIFICATE-----
      MIICqjCCAk+gAwIBAgIJAIuJ+0352Kq4MAoGCCqGSM49BAMCMIGwMQswCQYDVQQG
      EwJVUzETMBEGA1UECAwKV2FzaGluZ3RvbjERMA8GA1UEBwwIS2lya2xhbmQxFTAT
      BgNVBAoMDEdvb2dsZSwgSW5jLjEeMBwGA1UECwwVR29vZ2xlIENsb3VkIFBsYXRm
      b3JtMR8wHQYDVQQDDBZ3d3cubXktc2VjdXJlLXNpdGUuY29tMSEwHwYJKoZIhvcN
      AQkBFhJuZWxzb25hQGdvb2dsZS5jb20wHhcNMTcwNjI4MDQ1NjI2WhcNMjcwNjI2
      MDQ1NjI2WjCBsDELMAkGA1UEBhMCVVMxEzARBgNVBAgMCldhc2hpbmd0b24xETAP
      BgNVBAcMCEtpcmtsYW5kMRUwEwYDVQQKDAxHb29nbGUsIEluYy4xHjAcBgNVBAsM
      FUdvb2dsZSBDbG91ZCBQbGF0Zm9ybTEfMB0GA1UEAwwWd3d3Lm15LXNlY3VyZS1z
      aXRlLmNvbTEhMB8GCSqGSIb3DQEJARYSbmVsc29uYUBnb29nbGUuY29tMFkwEwYH
      KoZIzj0CAQYIKoZIzj0DAQcDQgAEHGzpcRJ4XzfBJCCPMQeXQpTXwlblimODQCuQ
      4mzkzTv0dXyB750fOGN02HtkpBOZzzvUARTR10JQoSe2/5PIwaNQME4wHQYDVR0O
      BBYEFKIQC3A2SDpxcdfn0YLKineDNq/BMB8GA1UdIwQYMBaAFKIQC3A2SDpxcdfn
      0YLKineDNq/BMAwGA1UdEwQFMAMBAf8wCgYIKoZIzj0EAwIDSQAwRgIhALs4vy+O
      M3jcqgA4fSW/oKw6UJxp+M6a+nGMX+UJR3YgAiEAvvl39QRVAiv84hdoCuyON0lJ
      zqGNhIPGq2ULqXKK8BY=
      -----END CERTIFICATE-----
    private_key: |-
      -----BEGIN EC PRIVATE KEY-----
      MHcCAQEEIObtRo8tkUqoMjeHhsOh2ouPpXCgBcP+EDxZCB/tws15oAoGCCqGSM49
      AwEHoUQDQgAEHGzpcRJ4XzfBJCCPMQeXQpTXwlblimODQCuQ4mzkzTv0dXyB750f
      OGN02HtkpBOZzzvUARTR10JQoSe2/5PIwQ==
      -----END EC PRIVATE KEY-----
    project: "{{ gcp_project }}"
    auth_kind: "{{ gcp_cred_kind }}"
    service_account_file: "{{ gcp_cred_file }}"
    state: present
  register: sslcert

- name: create a target HTTPS proxy
  gcp_compute_target_https_proxy:
    name: test_object
    ssl_certificates:
    - "{{ sslcert }}"
    url_map: "{{ urlmap }}"
    project: test_project
    auth_kind: serviceaccount
    service_account_file: "/tmp/auth.pem"
    state: present
'''

RETURN = '''
creationTimestamp:
  description:
  - Creation timestamp in RFC3339 text format.
  returned: success
  type: str
description:
  description:
  - An optional description of this resource.
  returned: success
  type: str
id:
  description:
  - The unique identifier for the resource.
  returned: success
  type: int
name:
  description:
  - Name of the resource. Provided by the client when the resource is created. The
    name must be 1-63 characters long, and comply with RFC1035. Specifically, the
    name must be 1-63 characters long and match the regular expression `[a-z]([-a-z0-9]*[a-z0-9])?`
    which means the first character must be a lowercase letter, and all following
    characters must be a dash, lowercase letter, or digit, except the last character,
    which cannot be a dash.
  returned: success
  type: str
quicOverride:
  description:
  - Specifies the QUIC override policy for this resource. This determines whether
    the load balancer will attempt to negotiate QUIC with clients or not. Can specify
    one of NONE, ENABLE, or DISABLE. If NONE is specified, uses the QUIC policy with
    no user overrides, which is equivalent to DISABLE. Not specifying this field is
    equivalent to specifying NONE.
  returned: success
  type: str
sslCertificates:
  description:
  - A list of SslCertificate resources that are used to authenticate connections between
    users and the load balancer. Currently, exactly one SSL certificate must be specified.
  returned: success
  type: list
sslPolicy:
  description:
  - A reference to the SslPolicy resource that will be associated with the TargetHttpsProxy
    resource. If not set, the TargetHttpsProxy resource will not have any SSL policy
    configured.
  returned: success
  type: dict
urlMap:
  description:
  - A reference to the UrlMap resource that defines the mapping from URL to the BackendService.
  returned: success
  type: dict
'''

################################################################################
# Imports
################################################################################

from ansible.module_utils.gcp_utils import navigate_hash, GcpSession, GcpModule, GcpRequest, replace_resource_dict
import json
import time

################################################################################
# Main
################################################################################


def main():
    """Main function"""

    module = GcpModule(
        argument_spec=dict(
            state=dict(default='present', choices=['present', 'absent'], type='str'),
            description=dict(type='str'),
            name=dict(required=True, type='str'),
            quic_override=dict(type='str'),
            ssl_certificates=dict(required=True, type='list', elements='dict'),
            ssl_policy=dict(type='dict'),
            url_map=dict(required=True, type='dict'),
        )
    )

    if not module.params['scopes']:
        module.params['scopes'] = ['https://www.googleapis.com/auth/compute']

    state = module.params['state']
    kind = 'compute#targetHttpsProxy'

    fetch = fetch_resource(module, self_link(module), kind)
    changed = False

    if fetch:
        if state == 'present':
            if is_different(module, fetch):
                update(module, self_link(module), kind, fetch)
                fetch = fetch_resource(module, self_link(module), kind)
                changed = True
        else:
            delete(module, self_link(module), kind)
            fetch = {}
            changed = True
    else:
        if state == 'present':
            fetch = create(module, collection(module), kind)
            changed = True
        else:
            fetch = {}

    fetch.update({'changed': changed})

    module.exit_json(**fetch)


def create(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.post(link, resource_to_request(module)))


def update(module, link, kind, fetch):
    update_fields(module, resource_to_request(module), response_to_hash(module, fetch))
    return fetch_resource(module, self_link(module), kind)


def update_fields(module, request, response):
    if response.get('quicOverride') != request.get('quicOverride'):
        quic_override_update(module, request, response)
    if response.get('sslCertificates') != request.get('sslCertificates'):
        ssl_certificates_update(module, request, response)
    if response.get('sslPolicy') != request.get('sslPolicy'):
        ssl_policy_update(module, request, response)
    if response.get('urlMap') != request.get('urlMap'):
        url_map_update(module, request, response)


def quic_override_update(module, request, response):
    auth = GcpSession(module, 'compute')
    auth.post(
        ''.join(["https://www.googleapis.com/compute/v1/", "projects/{project}/global/targetHttpsProxies/{name}/setQuicOverride"]).format(**module.params),
        {u'quicOverride': module.params.get('quic_override')},
    )


def ssl_certificates_update(module, request, response):
    auth = GcpSession(module, 'compute')
    auth.post(
        ''.join(["https://www.googleapis.com/compute/v1/", "projects/{project}/targetHttpsProxies/{name}/setSslCertificates"]).format(**module.params),
        {u'sslCertificates': replace_resource_dict(module.params.get('ssl_certificates', []), 'selfLink')},
    )


def ssl_policy_update(module, request, response):
    auth = GcpSession(module, 'compute')
    auth.post(
        ''.join(["https://www.googleapis.com/compute/v1/", "projects/{project}/global/targetHttpsProxies/{name}/setSslPolicy"]).format(**module.params),
        {u'sslPolicy': replace_resource_dict(module.params.get(u'ssl_policy', {}), 'selfLink')},
    )


def url_map_update(module, request, response):
    auth = GcpSession(module, 'compute')
    auth.post(
        ''.join(["https://www.googleapis.com/compute/v1/", "projects/{project}/targetHttpsProxies/{name}/setUrlMap"]).format(**module.params),
        {u'urlMap': replace_resource_dict(module.params.get(u'url_map', {}), 'selfLink')},
    )


def delete(module, link, kind):
    auth = GcpSession(module, 'compute')
    return wait_for_operation(module, auth.delete(link))


def resource_to_request(module):
    request = {
        u'kind': 'compute#targetHttpsProxy',
        u'description': module.params.get('description'),
        u'name': module.params.get('name'),
        u'quicOverride': module.params.get('quic_override'),
        u'sslCertificates': replace_resource_dict(module.params.get('ssl_certificates', []), 'selfLink'),
        u'sslPolicy': replace_resource_dict(module.params.get(u'ssl_policy', {}), 'selfLink'),
        u'urlMap': replace_resource_dict(module.params.get(u'url_map', {}), 'selfLink'),
    }
    return_vals = {}
    for k, v in request.items():
        if v or v is False:
            return_vals[k] = v

    return return_vals


def fetch_resource(module, link, kind, allow_not_found=True):
    auth = GcpSession(module, 'compute')
    return return_if_object(module, auth.get(link), kind, allow_not_found)


def self_link(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/global/targetHttpsProxies/{name}".format(**module.params)


def collection(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/global/targetHttpsProxies".format(**module.params)


def return_if_object(module, response, kind, allow_not_found=False):
    # If not found, return nothing.
    if allow_not_found and response.status_code == 404:
        return None

    # If no content, return nothing.
    if response.status_code == 204:
        return None

    try:
        module.raise_for_status(response)
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError):
        module.fail_json(msg="Invalid JSON response with error: %s" % response.text)

    if navigate_hash(result, ['error', 'errors']):
        module.fail_json(msg=navigate_hash(result, ['error', 'errors']))

    return result


def is_different(module, response):
    request = resource_to_request(module)
    response = response_to_hash(module, response)

    # Remove all output-only from response.
    response_vals = {}
    for k, v in response.items():
        if k in request:
            response_vals[k] = v

    request_vals = {}
    for k, v in request.items():
        if k in response:
            request_vals[k] = v

    return GcpRequest(request_vals) != GcpRequest(response_vals)


# Remove unnecessary properties from the response.
# This is for doing comparisons with Ansible's current parameters.
def response_to_hash(module, response):
    return {
        u'creationTimestamp': response.get(u'creationTimestamp'),
        u'description': module.params.get('description'),
        u'id': response.get(u'id'),
        u'name': module.params.get('name'),
        u'quicOverride': response.get(u'quicOverride'),
        u'sslCertificates': response.get(u'sslCertificates'),
        u'sslPolicy': response.get(u'sslPolicy'),
        u'urlMap': response.get(u'urlMap'),
    }


def async_op_url(module, extra_data=None):
    if extra_data is None:
        extra_data = {}
    url = "https://www.googleapis.com/compute/v1/projects/{project}/global/operations/{op_id}"
    combined = extra_data.copy()
    combined.update(module.params)
    return url.format(**combined)


def wait_for_operation(module, response):
    op_result = return_if_object(module, response, 'compute#operation')
    if op_result is None:
        return {}
    status = navigate_hash(op_result, ['status'])
    wait_done = wait_for_completion(status, op_result, module)
    return fetch_resource(module, navigate_hash(wait_done, ['targetLink']), 'compute#targetHttpsProxy')


def wait_for_completion(status, op_result, module):
    op_id = navigate_hash(op_result, ['name'])
    op_uri = async_op_url(module, {'op_id': op_id})
    while status != 'DONE':
        raise_if_errors(op_result, ['error', 'errors'], module)
        time.sleep(1.0)
        op_result = fetch_resource(module, op_uri, 'compute#operation', False)
        status = navigate_hash(op_result, ['status'])
    return op_result


def raise_if_errors(response, err_path, module):
    errors = navigate_hash(response, err_path)
    if errors is not None:
        module.fail_json(msg=errors)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2016, Adam Števko <adam.stevko@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = r'''
---
module: beadm
short_description: Manage ZFS boot environments on FreeBSD/Solaris/illumos systems.
description:
    - Create, delete or activate ZFS boot environments.
    - Mount and unmount ZFS boot environments.
version_added: "2.3"
author: Adam Števko (@xen0l)
options:
    name:
        description:
            - ZFS boot environment name.
        type: str
        required: True
        aliases: [ "be" ]
    snapshot:
        description:
            - If specified, the new boot environment will be cloned from the given
              snapshot or inactive boot environment.
        type: str
    description:
        description:
            - Associate a description with a new boot environment. This option is
              available only on Solarish platforms.
        type: str
    options:
        description:
            - Create the datasets for new BE with specific ZFS properties.
            - Multiple options can be specified.
            - This option is available only on Solarish platforms.
        type: str
    mountpoint:
        description:
            - Path where to mount the ZFS boot environment.
        type: path
    state:
        description:
            - Create or delete ZFS boot environment.
        type: str
        choices: [ absent, activated, mounted, present, unmounted ]
        default: present
    force:
        description:
            - Specifies if the unmount should be forced.
        type: bool
        default: false
'''

EXAMPLES = r'''
- name: Create ZFS boot environment
  beadm:
    name: upgrade-be
    state: present

- name: Create ZFS boot environment from existing inactive boot environment
  beadm:
    name: upgrade-be
    snapshot: be@old
    state: present

- name: Create ZFS boot environment with compression enabled and description "upgrade"
  beadm:
    name: upgrade-be
    options: "compression=on"
    description: upgrade
    state: present

- name: Delete ZFS boot environment
  beadm:
    name: old-be
    state: absent

- name: Mount ZFS boot environment on /tmp/be
  beadm:
    name: BE
    mountpoint: /tmp/be
    state: mounted

- name: Unmount ZFS boot environment
  beadm:
    name: BE
    state: unmounted

- name: Activate ZFS boot environment
  beadm:
    name: upgrade-be
    state: activated
'''

RETURN = r'''
name:
    description: BE name
    returned: always
    type: str
    sample: pre-upgrade
snapshot:
    description: ZFS snapshot to create BE from
    returned: always
    type: str
    sample: rpool/ROOT/oi-hipster@fresh
description:
    description: BE description
    returned: always
    type: str
    sample: Upgrade from 9.0 to 10.0
options:
    description: BE additional options
    returned: always
    type: str
    sample: compression=on
mountpoint:
    description: BE mountpoint
    returned: always
    type: str
    sample: /mnt/be
state:
    description: state of the target
    returned: always
    type: str
    sample: present
force:
    description: If forced action is wanted
    returned: always
    type: bool
    sample: False
'''

import os
import re
from ansible.module_utils.basic import AnsibleModule


class BE(object):
    def __init__(self, module):
        self.module = module

        self.name = module.params['name']
        self.snapshot = module.params['snapshot']
        self.description = module.params['description']
        self.options = module.params['options']
        self.mountpoint = module.params['mountpoint']
        self.state = module.params['state']
        self.force = module.params['force']
        self.is_freebsd = os.uname()[0] == 'FreeBSD'

    def _beadm_list(self):
        cmd = [self.module.get_bin_path('beadm')]
        cmd.append('list')
        cmd.append('-H')
        if '@' in self.name:
            cmd.append('-s')
        return self.module.run_command(cmd)

    def _find_be_by_name(self, out):
        if '@' in self.name:
            for line in out.splitlines():
                if self.is_freebsd:
                    check = re.match(r'.+/({0})\s+\-'.format(self.name), line)
                    if check:
                        return check
                else:
                    check = line.split(';')
                    if check[1] == self.name:
                        return check
        else:
            splitter = '\t' if self.is_freebsd else ';'
            for line in out.splitlines():
                check = line.split(splitter)
                if check[0] == self.name:
                    return check

        return None

    def exists(self):
        (rc, out, _) = self._beadm_list()

        if rc == 0:
            if self._find_be_by_name(out):
                return True
            else:
                return False
        else:
            return False

    def is_activated(self):
        (rc, out, _) = self._beadm_list()

        if rc == 0:
            line = self._find_be_by_name(out)
            if self.is_freebsd:
                if line is not None and 'R' in line.split('\t')[1]:
                    return True
            else:
                if 'R' in line.split(';')[2]:
                    return True

        return False

    def activate_be(self):
        cmd = [self.module.get_bin_path('beadm')]

        cmd.append('activate')
        cmd.append(self.name)

        return self.module.run_command(cmd)

    def create_be(self):
        cmd = [self.module.get_bin_path('beadm')]

        cmd.append('create')

        if self.snapshot:
            cmd.append('-e')
            cmd.append(self.snapshot)

        if not self.is_freebsd:
            if self.description:
                cmd.append('-d')
                cmd.append(self.description)

            if self.options:
                cmd.append('-o')
                cmd.append(self.options)

        cmd.append(self.name)

        return self.module.run_command(cmd)

    def destroy_be(self):
        cmd = [self.module.get_bin_path('beadm')]

        cmd.append('destroy')
        cmd.append('-F')
        cmd.append(self.name)

        return self.module.run_command(cmd)

    def is_mounted(self):
        (rc, out, _) = self._beadm_list()

        if rc == 0:
            line = self._find_be_by_name(out)
            if self.is_freebsd:
                # On FreeBSD, we exclude currently mounted BE on /, as it is
                # special and can be activated even if it is mounted. That is not
                # possible with non-root BEs.
                if line.split('\t')[2] != '-' and \
                        line.split('\t')[2] != '/':
                    return True
            else:
                if line.split(';')[3]:
                    return True

        return False

    def mount_be(self):
        cmd = [self.module.get_bin_path('beadm')]

        cmd.append('mount')
        cmd.append(self.name)

        if self.mountpoint:
            cmd.append(self.mountpoint)

        return self.module.run_command(cmd)

    def unmount_be(self):
        cmd = [self.module.get_bin_path('beadm')]

        cmd.append('unmount')
        if self.force:
            cmd.append('-f')
        cmd.append(self.name)

        return self.module.run_command(cmd)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type='str', required=True, aliases=['be']),
            snapshot=dict(type='str'),
            description=dict(type='str'),
            options=dict(type='str'),
            mountpoint=dict(type='path'),
            state=dict(type='str', default='present', choices=['absent', 'activated', 'mounted', 'present', 'unmounted']),
            force=dict(type='bool', default=False),
        ),
        supports_check_mode=True,
    )

    be = BE(module)

    rc = None
    out = ''
    err = ''
    result = {}
    result['name'] = be.name
    result['state'] = be.state

    if be.snapshot:
        result['snapshot'] = be.snapshot

    if be.description:
        result['description'] = be.description

    if be.options:
        result['options'] = be.options

    if be.mountpoint:
        result['mountpoint'] = be.mountpoint

    if be.state == 'absent':
        # beadm on FreeBSD and Solarish systems differs in delete behaviour in
        # that we are not allowed to delete activated BE on FreeBSD while on
        # Solarish systems we cannot delete BE if it is mounted. We add mount
        # check for both platforms as BE should be explicitly unmounted before
        # being deleted. On FreeBSD, we also check if the BE is activated.
        if be.exists():
            if not be.is_mounted():
                if module.check_mode:
                    module.exit_json(changed=True)

                if be.is_freebsd:
                    if be.is_activated():
                        module.fail_json(msg='Unable to remove active BE!')

                (rc, out, err) = be.destroy_be()

                if rc != 0:
                    module.fail_json(msg='Error while destroying BE: "%s"' % err,
                                     name=be.name,
                                     stderr=err,
                                     rc=rc)
            else:
                module.fail_json(msg='Unable to remove BE as it is mounted!')

    elif be.state == 'present':
        if not be.exists():
            if module.check_mode:
                module.exit_json(changed=True)

            (rc, out, err) = be.create_be()

            if rc != 0:
                module.fail_json(msg='Error while creating BE: "%s"' % err,
                                 name=be.name,
                                 stderr=err,
                                 rc=rc)

    elif be.state == 'activated':
        if not be.is_activated():
            if module.check_mode:
                module.exit_json(changed=True)

            # On FreeBSD, beadm is unable to activate mounted BEs, so we add
            # an explicit check for that case.
            if be.is_freebsd:
                if be.is_mounted():
                    module.fail_json(msg='Unable to activate mounted BE!')

            (rc, out, err) = be.activate_be()

            if rc != 0:
                module.fail_json(msg='Error while activating BE: "%s"' % err,
                                 name=be.name,
                                 stderr=err,
                                 rc=rc)
    elif be.state == 'mounted':
        if not be.is_mounted():
            if module.check_mode:
                module.exit_json(changed=True)

            (rc, out, err) = be.mount_be()

            if rc != 0:
                module.fail_json(msg='Error while mounting BE: "%s"' % err,
                                 name=be.name,
                                 stderr=err,
                                 rc=rc)

    elif be.state == 'unmounted':
        if be.is_mounted():
            if module.check_mode:
                module.exit_json(changed=True)

            (rc, out, err) = be.unmount_be()

            if rc != 0:
                module.fail_json(msg='Error while unmounting BE: "%s"' % err,
                                 name=be.name,
                                 stderr=err,
                                 rc=rc)

    if rc is None:
        result['changed'] = False
    else:
        result['changed'] = True

    if out:
        result['stdout'] = out
    if err:
        result['stderr'] = err

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for nxos_l2_interfaces
"""

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}

DOCUMENTATION = """
---
module: nxos_l2_interfaces
version_added: 2.9
short_description: Manages Layer-2 Interfaces attributes of NX-OS Interfaces
description: This module manages Layer-2 interfaces attributes of NX-OS Interfaces.
author: Trishna Guha (@trishnaguha)
notes:
  - Tested against NXOS 7.3.(0)D1(1) on VIRL
options:
  config:
    description: A dictionary of Layer-2 interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Full name of interface, i.e. Ethernet1/1.
        type: str
        required: true
      access:
        description:
          - Switchport mode access command to configure the interface as a
            Layer-2 access.
        type: dict
        suboptions:
          vlan:
            description:
            - Configure given VLAN in access port. It's used as the access
              VLAN ID.
            type: int
      trunk:
        description:
          - Switchport mode trunk command to configure the interface as a
            Layer-2 trunk.
        type: dict
        suboptions:
          native_vlan:
            description:
              - Native VLAN to be configured in trunk port. It is used as the
                trunk native VLAN ID.
            type: int
          allowed_vlans:
            description:
              - List of allowed VLANs in a given trunk port. These are the only
                VLANs that will be configured on the trunk.
            type: str

  state:
    description:
      - The state of the configuration after module completion.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
    default: merged
"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# interface Ethernet1/1
#   switchport access vlan 20
# interface Ethernet1/2
#   switchport trunk native vlan 20
# interface mgmt0
#   ip address dhcp
#   ipv6 address auto-config

- name: Merge provided configuration with device configuration.
  nxos_l2_interfaces:
    config:
      - name: Ethernet1/1
        trunk:
          native_vlan: 10
          allowed_vlans: 2,4,15
      - name: Ethernet1/2
        access:
          vlan: 30
    state: merged

# After state:
# ------------
#
# interface Ethernet1/1
#   switchport trunk native vlan 10
#   switchport trunk allowed vlans 2,4,15
# interface Ethernet1/2
#   switchport access vlan 30
# interface mgmt0
#   ip address dhcp
#   ipv6 address auto-config


# Using replaced

# Before state:
# -------------
#
# interface Ethernet1/1
#   switchport access vlan 20
# interface Ethernet1/2
#   switchport trunk native vlan 20
# interface mgmt0
#   ip address dhcp
#   ipv6 address auto-config

- name: Replace device configuration of specified L2 interfaces with provided configuration.
  nxos_l2_interfaces:
    config:
      - name: Ethernet1/1
        trunk:
          native_vlan: 20
          trunk_vlans: 5-10, 15
    state: replaced

# After state:
# ------------
#
# interface Ethernet1/1
#   switchport trunk native vlan 20
#   switchport trunk allowed vlan 5-10,15
# interface Ethernet1/2
#   switchport trunk native vlan 20
#   switchport mode trunk
# interface mgmt0
#   ip address dhcp
#   ipv6 address auto-config


# Using overridden

# Before state:
# -------------
#
# interface Ethernet1/1
#   switchport access vlan 20
# interface Ethernet1/2
#   switchport trunk native vlan 20
# interface mgmt0
#   ip address dhcp
#   ipv6 address auto-config

- name: Override device configuration of all L2 interfaces on device with provided configuration.
  nxos_l2_interfaces:
    config:
      - name: Ethernet1/2
        access:
          vlan: 30
    state: overridden

# After state:
# ------------
#
# interface Ethernet1/1
# interface Ethernet1/2
#   switchport access vlan 30
# interface mgmt0
#   ip address dhcp
#   ipv6 address auto-config


# Using deleted

# Before state:
# -------------
#
# interface Ethernet1/1
#   switchport access vlan 20
# interface Ethernet1/2
#   switchport trunk native vlan 20
# interface mgmt0
#   ip address dhcp
#   ipv6 address auto-config

- name: Delete L2 attributes of given interfaces (Note This won't delete the interface itself).
  nxos_l2_interfaces:
    config:
      - name: Ethernet1/1
      - name: Ethernet1/2
    state: deleted

# After state:
# ------------
#
# interface Ethernet1/1
# interface Ethernet1/2
# interface mgmt0
#   ip address dhcp
#   ipv6 address auto-config


"""
RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['command 1', 'command 2', 'command 3']
"""


from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.nxos.argspec.l2_interfaces.l2_interfaces import L2_interfacesArgs
from ansible.module_utils.network.nxos.config.l2_interfaces.l2_interfaces import L2_interfaces


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(argument_spec=L2_interfacesArgs.argument_spec,
                           supports_check_mode=True)

    result = L2_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == '__main__':
    main()
